import unittest

if not hasattr(unittest.TestCase, "assertIn"):
    import unittest2 as unittest

try:
    import simplejson as json
except ImportError:
    import json

import cherrypy
from cherrypy import HTTPError, HTTPRedirect, InternalRedirect
from cherrypy.test import helper

from sqlalchemy import Column, Integer, Unicode, engine_from_config
from sqlalchemy.ext.declarative import declarative_base

from testconfig import config as testconfig

from blueberrypy.plugins import SQLAlchemyPlugin
from blueberrypy.tools import MultiHookPointTool, SQLAlchemySessionTool


def get_config(section_name):
    return dict([(str(k), v) for k, v in testconfig[section_name].iteritems()])

Base = declarative_base()


class User(Base):
    __tablename__ = 'user'
    id = Column(Integer, autoincrement=True, primary_key=True)
    name = Column(Unicode(128), nullable=False, unique=True)


class Address(Base):
    __tablename__ = 'address'
    id = Column(Integer, autoincrement=True, primary_key=True)
    address = Column(Unicode(128), nullable=False, unique=True)


class MultiHookPointToolTest(helper.CPWebCase, unittest.TestCase):

    @staticmethod
    def setup_server():

        class TestMultiHookPointTool(MultiHookPointTool):

            def on_start_resource(self, param=None):
                req = cherrypy.request
                req.on_start_resource_param = param

            def before_request_body(self, param=None):
                req = cherrypy.request
                req.before_request_body_param = param

            def before_handler(self, param=None):
                req = cherrypy.request
                req.before_handler_param = param

            def before_finalize(self, param=None):
                resp = cherrypy.response
                resp.headers['x-before-finalize'] = param

            def on_end_resource(self, param=None):
                resp = cherrypy.response
                resp.header_list += ('x-on-end-resource', str(param)),

            def before_error_response(self, param=None):
                resp = cherrypy.response
                resp.headers['x-before-error-response'] = param

            def after_error_response(self, param=None):
                resp = cherrypy.response
                resp.headers['x-after-error-response'] = param

            def on_end_request(self, param=None):
                MultiHookPointToolTest.on_end_request_param = param

        class Root(object):

            _cp_config = {"tools.test_multi_hook_point.on": True}

            @cherrypy.expose
            def success(self):
                req = cherrypy.request
                return str(req.on_start_resource_param | req.before_request_body_param | req.before_handler_param)
            success._cp_config = {"tools.test_multi_hook_point.on_start_resource.param": 1,
                                  "tools.test_multi_hook_point.before_request_body.param": 2,
                                  "tools.test_multi_hook_point.before_handler.param": 4,
                                  "tools.test_multi_hook_point.before_finalize.param": 11,
                                  "tools.test_multi_hook_point.on_end_resource.param": 13,
                                  "tools.test_multi_hook_point.on_end_request.param": 19}

            @cherrypy.expose
            def failure(self):
                return 1 / 0
            failure._cp_config = {"tools.test_multi_hook_point.before_error_response.param": 7,
                                  "tools.test_multi_hook_point.after_error_response.param": 8}

        cherrypy.tools.test_multi_hook_point = TestMultiHookPointTool()
        cherrypy.tree.mount(Root())

    def test_success(self):
        self.getPage("/success")
        self.assertInBody(str(7))
        self.assertHeader("x-before-finalize", 11)
        self.assertHeader("x-on-end-resource", 13)
        self.assertEqual(MultiHookPointToolTest.on_end_request_param, 19)

    def test_failure(self):
        self.getPage("/failure")
        self.assertHeader("x-before-error-response", 7)
        self.assertHeader("x-after-error-response", 8)


class SQLAlchemySessionToolSingleEngineTest(helper.CPWebCase, unittest.TestCase):

    engine = engine_from_config(get_config('sqlalchemy_engine'), '')

    @classmethod
    def setup_class(cls):

        super(SQLAlchemySessionToolSingleEngineTest, cls).setup_class()

        Base.metadata.create_all(cls.engine)
    setUpClass = setup_class

    @classmethod
    def teardown_class(cls):

        super(SQLAlchemySessionToolSingleEngineTest, cls).teardown_class()

        Base.metadata.drop_all(cls.engine)
    tearDownClass = teardown_class

    @staticmethod
    def setup_server():
        class SingleEngine(object):

            _cp_config = {'tools.orm_session.on': True}

            def save_user_and_address(self):
                session = cherrypy.request.orm_session
                bob = User(name=u"joey")
                session.add(bob)
                hk = Address(address=u"United States")
                session.add(hk)
                session.commit()
            save_user_and_address.exposed = True

            def query_user(self):
                session = cherrypy.request.orm_session
                joey = session.query(User).filter_by(name=u'joey').one()
                assert isinstance(joey.name, unicode)
                return json.dumps({'id': joey.id, 'name': joey.name})
            query_user.exposed = True

            def query_addresss(self):
                session = cherrypy.request.orm_session
                us = session.query(Address).filter_by(address=u'United States').one()
                assert isinstance(us.address, unicode)
                return json.dumps({'id': us.id, 'address': us.address})
            query_addresss.exposed = True

            def raise_not_passable_exception_save(self):
                session = cherrypy.request.orm_session
                bob = User(name=u"bob")
                session.add(bob)
                raise ValueError
            raise_not_passable_exception_save.exposed = True

            def raise_not_passable_exception_query(self):
                session = cherrypy.request.orm_session
                bob = session.query(User).filter_by(name=u'bob').first()
                return json.dumps(None)
            raise_not_passable_exception_query.exposed = True

            def raise_passable_exception_save(self):
                session = cherrypy.request.orm_session
                bob = User(name=u"bob")
                session.add(bob)
                session.commit()
                raise HTTPRedirect('/')
            raise_passable_exception_save.exposed = True

            def raise_passable_exception_query(self):
                session = cherrypy.request.orm_session
                bob = session.query(User).filter_by(name=u'bob').first()
                return json.dumps({'id': bob.id, 'name': bob.name})
            raise_passable_exception_query.exposed = True

        cherrypy.engine.sqlalchemy = SQLAlchemyPlugin(cherrypy.engine, testconfig)
        cherrypy.tools.orm_session = SQLAlchemySessionTool()
        cherrypy.config.update({'engine.sqlalchemy.on': True})
        cherrypy.tree.mount(SingleEngine())

    def test_orm_session_tool_commit(self):
        self.getPage('/save_user_and_address')
        self.assertStatus(200)

    def test_orm_session_tool_query_user(self):
        self.getPage('/query_user')
        json_resp = json.loads(self.body)
        self.assertIn('id', json_resp)
        self.assertIn('name', json_resp)
        self.assertEqual(u'joey', json_resp['name'])
        self.assertStatus(200)

    def test_orm_session_tool_query_address(self):
        self.getPage('/query_addresss')
        json_body = json.loads(self.body)
        self.assertIn('id', json_body)
        self.assertIn('address', json_body)
        self.assertEqual(u'United States', json_body['address'])
        self.assertStatus(200)

    def test_raise_not_passable_exception(self):
        self.getPage('/raise_not_passable_exception_save')
        self.assertStatus(500)
        self.getPage('/raise_not_passable_exception_query')
        self.assertBody(json.dumps(None))
        self.assertStatus(200)

    def test_raise_passable_exception(self):
        self.getPage('/raise_passable_exception_save')
        self.assertStatus(303)
        self.getPage('/raise_passable_exception_query')
        json_resp = json.loads(self.body)
        self.assertIn('id', json_resp)
        self.assertIn('name', json_resp)
        self.assertEqual(u'bob', json_resp['name'])
        self.assertStatus(200)


class SQLAlchemySessionToolTwoPhaseTest(helper.CPWebCase, unittest.TestCase):

    # only used for setup and teardown
    engine_bindings = {User: engine_from_config(get_config('sqlalchemy_engine_tests.test_tools.User'), ''),
                       Address: engine_from_config(get_config('sqlalchemy_engine_tests.test_tools.Address'), '')}

    @classmethod
    def setup_class(cls):

        super(SQLAlchemySessionToolTwoPhaseTest, cls).setup_class()

        User.metadata.create_all(cls.engine_bindings[User])
        Address.metadata.create_all(cls.engine_bindings[Address])
    setUpClass = setup_class

    @classmethod
    def teardown_class(cls):

        super(SQLAlchemySessionToolTwoPhaseTest, cls).teardown_class()

        User.metadata.create_all(cls.engine_bindings[User])
        Address.metadata.create_all(cls.engine_bindings[Address])
    tearDownClass = teardown_class

    @staticmethod
    def setup_server():

        class TwoPhase(object):

            _cp_config = {'tools.orm_session.on': True}

            def save_user_and_address(self):
                session = cherrypy.request.orm_session
                alice = User(name=u"alice")
                session.add(alice)
                hk = Address(address=u"Hong Kong")
                session.add(hk)
                session.commit()
            save_user_and_address.exposed = True
            save_user_and_address._cp_config = {'tools.orm_session.bindings': [User, Address]}

            def query_user(self):
                session = cherrypy.request.orm_session
                alice = session.query(User).filter_by(name=u'alice').one()
                assert isinstance(alice.name, unicode)
                return json.dumps({'id': alice.id, 'name': alice.name})
            query_user.exposed = True
            query_user._cp_config = {'tools.orm_session.bindings': [User]}

            def query_addresss(self):
                session = cherrypy.request.orm_session
                hk = session.query(Address).filter_by(address=u'Hong Kong').one()
                assert isinstance(hk.address, unicode)
                return json.dumps({'id': hk.id, 'address': hk.address})
            query_addresss.exposed = True
            query_addresss._cp_config = {'tools.orm_session.bindings': [Address]}

            def raise_not_passable_exception_save(self):
                session = cherrypy.request.orm_session
                katy = User(name=u"katy")
                session.add(katy)
                raise HTTPError(400)
            raise_not_passable_exception_save.exposed = True
            raise_not_passable_exception_save._cp_config = {'tools.orm_session.bindings': [User]}

            def raise_not_passable_exception_query(self):
                session = cherrypy.request.orm_session
                katy = session.query(User).filter_by(name=u'katy').first()
                if katy is not None:
                    return json.dumps({'id': katy.id, 'name': katy.name})
                else:
                    return json.dumps(None)
            raise_not_passable_exception_query.exposed = True
            raise_not_passable_exception_query._cp_config = {'tools.orm_session.bindings': [User]}

            def raise_passable_exception_save(self):
                session = cherrypy.request.orm_session
                david = User(name=u"david")
                session.add(david)
                session.commit()
                raise HTTPRedirect('/')
            raise_passable_exception_save.exposed = True
            raise_passable_exception_save._cp_config = {'tools.orm_session.bindings': [User]}

            def raise_passable_exception_query(self):
                session = cherrypy.request.orm_session
                david = session.query(User).filter_by(name=u'david').first()
                return json.dumps({'id': david.id, 'name': david.name})
            raise_passable_exception_query.exposed = True
            raise_passable_exception_query._cp_config = {'tools.orm_session.bindings': [User]}

        cherrypy.engine.sqlalchemy = SQLAlchemyPlugin(cherrypy.engine, testconfig)
        cherrypy.tools.orm_session = SQLAlchemySessionTool()
        cherrypy.config.update({'engine.sqlalchemy.on': True})
        cherrypy.tree.mount(TwoPhase())

    def test_orm_session_tool_commit(self):
        self.getPage('/save_user_and_address')
        self.assertStatus(200)

    def test_orm_session_tool_query_user(self):
        self.getPage('/query_user')
        json_resp = json.loads(self.body)
        self.assertIn('id', json_resp)
        self.assertIn('name', json_resp)
        self.assertEqual(u'alice', json_resp['name'])
        self.assertStatus(200)

    def test_orm_session_tool_query_address(self):
        self.getPage('/query_addresss')
        json_body = json.loads(self.body)
        self.assertIn('id', json_body)
        self.assertIn('address', json_body)
        self.assertEqual(u'Hong Kong', json_body['address'])
        self.assertStatus(200)

    def test_raise_not_passable_exception(self):
        self.getPage('/raise_not_passable_exception_save')
        self.assertStatus(400)
        self.getPage('/raise_not_passable_exception_query')
        self.assertBody(json.dumps(None))
        self.assertStatus(200)

    def test_raise_passable_exception(self):
        self.getPage('/raise_passable_exception_save')
        self.assertStatus(303)
        self.getPage('/raise_passable_exception_query')
        json_resp = json.loads(self.body)
        self.assertIn('id', json_resp)
        self.assertIn('name', json_resp)
        self.assertEqual(u'david', json_resp['name'])
        self.assertStatus(200)



import cherrypy

from cherrypy.test.helper import CPWebCase

from blueberrypy.config import BlueberryPyConfiguration
from blueberrypy import email
from blueberrypy.plugins import LoggingPlugin
from blueberrypy.session import RedisSession
from blueberrypy.plugins import SQLAlchemyPlugin
from blueberrypy.tools import SQLAlchemySessionTool
from blueberrypy.template_engine import configure_jinja2


from testconfig import config as testconfig


class ControllerTestCase(CPWebCase):

    @staticmethod
    def setup_server():

        config = BlueberryPyConfiguration(app_config=testconfig)

        if config.use_email and config.email_config:
            email.configure(config.email_config)

        if config.use_logging and config.logging_config:
            cherrypy.engine.logging = LoggingPlugin(cherrypy.engine,
                                                    config=config.logging_config)

        if config.use_redis:
            cherrypy.lib.sessions.RedisSession = RedisSession

        if config.use_sqlalchemy:
            cherrypy.engine.sqlalchemy = SQLAlchemyPlugin(cherrypy.engine,
                                                          config=config.sqlalchemy_config)
            cherrypy.tools.orm_session = SQLAlchemySessionTool()

        if config.use_jinja2:
            if config.webassets_env:
                configure_jinja2(assets_env=config.webassets_env,
                                        **config.jinja2_config)
            else:
                configure_jinja2(**config.jinja2_config)

        cherrypy.config.update(config.app_config)

        # mount the controllers
        for script_name, section in config.controllers_config.iteritems():
            section = section.copy()
            controller = section.pop("controller")
            if isinstance(controller, cherrypy.dispatch.RoutesDispatcher):
                routes_config = {'/': {"request.dispatch": controller}}
                for path in section.iterkeys():
                    if path.strip() == '/':
                        routes_config['/'].update(section['/'])
                    else:
                        routes_config[path] = section[path].copy()
                app_config = config.app_config.copy()
                app_config.pop("controllers")
                routes_config.update(app_config)
                cherrypy.tree.mount(None, script_name=script_name,
                                    config=routes_config)
            else:
                app_config = config.app_config.copy()
                app_config.pop("controllers")
                controller_config = section.copy()
                controller_config.update(app_config)
                cherrypy.tree.mount(controller(), script_name=script_name,
                                    config=controller_config)




import logging
import warnings

import cherrypy
from cherrypy._cptools import Tool, _getargs

from sqlalchemy.orm import scoped_session, sessionmaker
from sqlalchemy.exc import SQLAlchemyError


__all__ = ["SQLAlchemySessionTool"]


logger = logging.getLogger(__name__)


class MultiHookPointTool(Tool):
    """MultiHookPointTool provides subclasses the infrastructure for writing
    Tools that need to run at more than one request hook point.
    
    Subclasses can simply provide methods with the same name as the hook points
    and MuiltiHookPointTool will automatically wire each hook up to the request
    hook points when turned on in the configuration.
    """

    def __init__(self, name=None, priority=50):
        """Subclasses of MultiHookPointTool do not need to provide a default
        hook point and a callable because there's no such thing. As such,
        subclasses will most likely only provide a default priority for the
        hook point callables the subclasses provide.
        
        Same as the Tool class CherryPy supplies, the `name` parameter is set
        automatically when a MultiHookPointTool instance is attached to a
        ToolBox if not supplied.
        
        Lastly, similar to Tool, this constructor will also loop through all
        the hooks and attach their arguments directly to the Tool instance.
        The only difference is all the tool arguments are prefixed with their
        hook point names to avoid name conflicts.
        
        Example::
            
            app_config = {
                "/": {
                    "tools.my_multipoint_tool.on": True,
                    "tools.my_multipoint_tools.before_handler.echo": True,
                    "tools.my_multipoint_tools.before_finalize.priority": 80
                }
            }
        """
        self._name = name
        self._priority = priority
        self._setargs()

    def _setargs(self):
        for hook_point in cherrypy._cprequest.hookpoints:
            if hasattr(self, hook_point):
                hook = getattr(self, hook_point)
                try:
                    for arg in _getargs(hook):
                        setattr(self, hook_point + "_" + arg, None)
                # IronPython 1.0 raises NotImplementedError because
                # inspect.getargspec tries to access Python bytecode
                # in co_code attribute.
                except NotImplementedError:
                    pass
                # IronPython 1B1 may raise IndexError in some cases,
                # but if we trap it here it doesn't prevent CP from working.
                except IndexError:
                    pass

    def _setup(self):

        request = cherrypy.request

        conf = self._merged_args()

        for hook_point in cherrypy._cprequest.hookpoints:

            if hasattr(self, hook_point):
                hook = getattr(self, hook_point)
                if not callable(hook):
                    warnings.warn("%r is not a callable." % hook)

                hook_conf = {}
                for k, v in conf.iteritems():
                    if k.startswith(hook_point):
                        k = k.replace(hook_point, "").split(".", 1)[-1]
                        hook_conf[k] = v

                priority_key = hook_point + ".priority"
                hook_priority = self._priority if priority_key not in hook_conf else hook_conf[priority_key]
                request.hooks.attach(hook_point, hook, hook_priority, **hook_conf)

    def __call__(self, *args, **kwargs):
        raise NotImplementedError("This %r instance cannot be called directly." % \
            self.__class__.__name__)


class SQLAlchemySessionTool(MultiHookPointTool):
    """A CherryPy tool to process SQLAlchemy ORM sessions for requests.

    This tools sets up a scoped, possibly multi-engine SQLAlchemy ORM session to
    `cherrypy.request.orm_session` at the beginning of a request. This tool does
    not commit changes for you automatically, you must do you explicitly inside
    your controller code. At the end of each requests, this tool will rollback
    if errors occured. The session is guaranteed to be removed from the request
    in the end.

    As this tool hooks up _3_ callables to the request, this tools will also
    accept 3 `priority` options - `on_start_resource.priority`, 
    `before_finalize.priority` and `after_error_response.priority`. The `priority`
    option is still accepted as a default for all 3 hook points.
    """

    def on_start_resource(self, bindings=None):
        if bindings:

            if len(bindings) > 1:
                Session = scoped_session(sessionmaker(twophase=True))
            else:
                Session = scoped_session(sessionmaker())

            session_bindings = {}
            engine_bindings = cherrypy.engine.sqlalchemy.engine_bindings
            for binding in bindings:
                session_bindings[binding] = engine_bindings[binding]

            Session.configure(binds=session_bindings)

        else:
            Session = scoped_session(sessionmaker())
            Session.configure(bind=cherrypy.engine.sqlalchemy.engine)

        cherrypy.request.orm_session = Session

    def before_finalize(self):
        req = cherrypy.request
        session = req.orm_session
        session.remove()

    def after_error_response(self):
        req = cherrypy.request
        session = req.orm_session

        try:
            session.rollback()
            session.expunge_all()
        except SQLAlchemyError, e:
            logger.error(e, exc_info=True)
            cherrypy.log.error(msg=e, severity=logging.ERROR, traceback=True)
        finally:
            session.remove()



# -*- coding: utf-8 -*-

import unittest
import socket
import threading
import time

import cherrypy
from cherrypy.lib import sessions
from cherrypy.test import helper

from blueberrypy.session import RedisSession


def http_methods_allowed(methods=['GET', 'HEAD']):
    method = cherrypy.request.method.upper()
    if method not in methods:
        cherrypy.response.headers['Allow'] = ", ".join(methods)
        raise cherrypy.HTTPError(405)

cherrypy.tools.allow = cherrypy.Tool('on_start_resource', http_methods_allowed)


def setup_server():

    class Root:

        _cp_config = {'tools.sessions.on': True,
                      'tools.sessions.storage_type': 'redis',
                      'tools.sessions.host': 'localhost',
                      'tools.sessions.port': 6379,
                      'tools.sessions.prefix': 'whatever:',
                      'tools.sessions.timeout': (1.0 / 60)
                      }

        def clear(self):
            cherrypy.session.cache.flushdb()
        clear.exposed = True

        def data(self):
            cherrypy.session['aha'] = u'褔'
            return repr(cherrypy.session._data)
        data.exposed = True

        def testGen(self):
            counter = cherrypy.session.get('counter', 0) + 1
            cherrypy.session['counter'] = counter
            yield str(counter)
        testGen.exposed = True

        def testStr(self):
            counter = cherrypy.session.get('counter', 0) + 1
            cherrypy.session['counter'] = counter
            return str(counter)
        testStr.exposed = True

        def index(self):
            sess = cherrypy.session
            c = sess.get('counter', 0) + 1
            time.sleep(0.01)
            sess['counter'] = c
            return str(c)
        index.exposed = True

        def keyin(self, key):
            return str(key in cherrypy.session)
        keyin.exposed = True

        def delete(self):
            cherrypy.session.delete()
            sessions.expire()
            return "done"
        delete.exposed = True

        def delkey(self, key):
            del cherrypy.session[key]
            return "OK"
        delkey.exposed = True

        def blah(self):
            return self._cp_config['tools.sessions.storage_type']
        blah.exposed = True

        def iredir(self):
            raise cherrypy.InternalRedirect('/blah')
        iredir.exposed = True

        def restricted(self):
            return cherrypy.request.method
        restricted.exposed = True
        restricted._cp_config = {'tools.allow.on': True,
                                 'tools.allow.methods': ['GET']}

        def regen(self):
            cherrypy.tools.sessions.regenerate()
            return "logged in"
        regen.exposed = True

        def length(self):
            return str(len(cherrypy.session))
        length.exposed = True

        def session_cookie(self):
            # Must load() to start the clean thread.
            cherrypy.session.load()
            return cherrypy.session.id
        session_cookie.exposed = True
        session_cookie._cp_config = {
            'tools.sessions.path': '/session_cookie',
            'tools.sessions.name': 'temp',
            'tools.sessions.persistent': False}

    sessions.RedisSession = RedisSession
    cherrypy.tree.mount(Root())

# testing that redis-py is available and that we have a redis server running
try:
    import redis

    host, port = '127.0.0.1', 6379
    for res in socket.getaddrinfo(host, port, socket.AF_UNSPEC,
                                  socket.SOCK_STREAM):
        af, socktype, proto, canonname, sa = res
        s = None
        try:
            s = socket.socket(af, socktype, proto)
            s.settimeout(1.0)
            s.connect((host, port))
            s.close()
        except socket.error:
            if s:
                s.close()
            raise
        break

except ImportError:

    class RedisSessionTest(unittest.TestCase):
        def test_nothing(self):
            self.fail("redis-py not available")

except socket.error:

    class RedisSessionTest(unittest.TestCase):
        def test_nothing(self):
            self.fail("redis not reachable")

else:
    class RedisSessionTest(helper.CPWebCase):

        setup_server = staticmethod(setup_server)

        def test_0_Session(self):
            self.getPage('/testStr')
            self.assertBody('1')
            self.getPage('/testGen', self.cookies)
            self.assertBody('2')
            self.getPage('/testStr', self.cookies)
            self.assertBody('3')
            self.getPage('/length', self.cookies)
            self.assertBody('1')
            self.getPage('/delkey?key=counter', self.cookies)
            self.assertStatus(200)

            # Wait for the session.timeout (1 second)
            time.sleep(1.25)
            self.getPage('/')
            self.assertBody('1')

            # Test session __contains__
            self.getPage('/keyin?key=counter', self.cookies)
            self.assertBody("True")

            # Test session delete
            self.getPage('/delete', self.cookies)
            self.assertBody("done")

        def test_1_Concurrency(self):
            client_thread_count = 5
            request_count = 30

            # Get initial cookie
            self.getPage("/")
            self.assertBody("1")
            cookies = self.cookies

            data_dict = {}

            def request(index):
                for i in range(request_count):
                    self.getPage("/", cookies)
                    # Uncomment the following line to prove threads overlap.
##                    sys.stdout.write("%d " % index)
                if not self.body.isdigit():
                    self.fail(self.body)
                data_dict[index] = v = int(self.body)

            # Start <request_count> concurrent requests from
            # each of <client_thread_count> clients
            ts = []
            for c in range(client_thread_count):
                data_dict[c] = 0
                t = threading.Thread(target=request, args=(c,))
                ts.append(t)
                t.start()

            for t in ts:
                t.join()

            hitcount = max(data_dict.values())
            expected = 1 + (client_thread_count * request_count)
            self.assertEqual(hitcount, expected)

        def test_3_Redirect(self):
            # Start a new session
            self.getPage('/testStr')
            self.getPage('/iredir', self.cookies)
            self.assertBody("redis")

        def test_5_Error_paths(self):
            self.getPage('/unknown/page')
            self.assertErrorPage(404, "The path '/unknown/page' was not found.")

            # Note: this path is *not* the same as above. The above
            # takes a normal route through the session code; this one
            # skips the session code's before_handler and only calls
            # before_finalize (save) and on_end (close). So the session
            # code has to survive calling save/close without init.
            self.getPage('/restricted', self.cookies, method='POST')
            self.assertErrorPage(405)




import logging
import unittest
import warnings

from email.header import decode_header

from lazr.smtptest.controller import QueueController

from blueberrypy import email
from blueberrypy.email import Mailer


logger = logging.getLogger(__name__)


class MailerTest(unittest.TestCase):

    def setUp(self):
        self.controller = QueueController("localhost", 9025)
        self.controller.start()

    def tearDown(self):
        self.controller.stop()

    def test_send_email(self):
        mailer = Mailer("localhost", 9025)
        body = "This is the bloody test body"
        mailer.send_email("rcpt@example.com", "from@example.com", "test subject", body)

        messages = list(self.controller)
        message = messages[0]
        (from_str, from_cs) = decode_header(message["From"])[0]
        (to_str, to_cs) = decode_header(message["To"])[0]
        (subject_str, subject_cs) = decode_header(message["Subject"])[0]

        self.assertEqual("from@example.com", from_str)
        self.assertEqual("rcpt@example.com", to_str)
        self.assertEqual("test subject", subject_str)
        self.assertEqual(body, message.get_payload(decode=True))

    def test_send_html_email(self):
        mailer = Mailer("localhost", 9025)
        text = u"This is the bloody test body"
        html = u"<p>This is the bloody test body</p>"
        mailer.send_html_email("rcpt@example.com", "from@example.com", "test subject", text, html)

        messages = list(self.controller)
        message = messages[0]
        (from_str, from_cs) = decode_header(message["From"])[0]
        (to_str, to_cs) = decode_header(message["To"])[0]
        (subject_str, subject_cs) = decode_header(message["Subject"])[0]

        self.assertEqual("from@example.com", from_str)
        self.assertEqual("rcpt@example.com", to_str)
        self.assertEqual("test subject", subject_str)
        self.assertEqual(text, message.get_payload(0).get_payload(decode=True))
        self.assertEqual("text/plain", message.get_payload(0).get_content_type())
        self.assertEqual(html, message.get_payload(1).get_payload(decode=True))
        self.assertEqual("text/html", message.get_payload(1).get_content_type())

class EmailModuleFuncTest(unittest.TestCase):

    def setUp(self):
        self.controller = QueueController("localhost", 9025)
        self.controller.start()

    def tearDown(self):
        self.controller.stop()

    def test_warnings(self):

        email._mailer = None

        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("error")
            self.assertRaises(UserWarning, email.send_email, "rcpt@example.com", "from@example.com", "test subject", "test body")
            self.assertRaises(UserWarning, email.send_html_email, "rcpt@example.com", "from@example.com", "test subject", "plain body", "<p>html body</p>")

    def test_send_email(self):
        email.configure({"host": "localhost",
                         "port": 9025})

        body = "This is the bloody test body"
        email.send_email("rcpt@example.com", "from@example.com", "test subject", body)

        messages = list(self.controller)
        message = messages[0]
        (from_str, from_cs) = decode_header(message["From"])[0]
        (to_str, to_cs) = decode_header(message["To"])[0]
        (subject_str, subject_cs) = decode_header(message["Subject"])[0]

        self.assertEqual("from@example.com", from_str)
        self.assertEqual("rcpt@example.com", to_str)
        self.assertEqual("test subject", subject_str)
        self.assertEqual(body, message.get_payload(decode=True))

    def test_send_html_email(self):
        email.configure({"host": "localhost",
                         "port": 9025})

        text = u"This is the bloody test body"
        html = u"<p>This is the bloody test body</p>"
        email.send_html_email("rcpt@example.com", "from@example.com", "test subject", text, html)

        messages = list(self.controller)
        message = messages[0]
        (from_str, from_cs) = decode_header(message["From"])[0]
        (to_str, to_cs) = decode_header(message["To"])[0]
        (subject_str, subject_cs) = decode_header(message["Subject"])[0]

        self.assertEqual("from@example.com", from_str)
        self.assertEqual("rcpt@example.com", to_str)
        self.assertEqual("test subject", subject_str)
        self.assertEqual(text, message.get_payload(0).get_payload(decode=True))
        self.assertEqual("text/plain", message.get_payload(0).get_content_type())
        self.assertEqual(html, message.get_payload(1).get_payload(decode=True))
        self.assertEqual("text/html", message.get_payload(1).get_content_type())



import os

from cherrypy.test import helper


class SQLAlchemyPluginTest(helper.CPWebCase):

    def test_engine_bindings(self):
        if os.name not in ['posix']:
            return self.skip("skipped (not on posix) ")

        p = helper.CPProcess(ssl=(self.scheme.lower() == 'https'))
        p.write_conf(extra='test_case_name: "test_engine_bindings"')
        p.start(imports='tests._test_plugins_engine_bindings')

        try:
            self.getPage("/engine_bindings")
            self.assertStatus(200)
            self.assertEqual("[(<class 'tests._test_plugins_engine_bindings.User'>, Engine(sqlite://)), (<class 'tests._test_plugins_engine_bindings.Group'>, Engine(sqlite://))]", self.body)
        finally:
            self.getPage("/exit")
        p.join()

    def test_engine(self):
        if os.name not in ['posix']:
            return self.skip("skipped (not on posix) ")

        p = helper.CPProcess(ssl=(self.scheme.lower() == 'https'))
        p.write_conf(extra='test_case_name: "test_engine"')
        p.start(imports='tests._test_plugins_engine')

        try:
            self.getPage("/engine")
            self.assertStatus(200)
            self.assertEqual(r"Engine(sqlite://)", self.body)
        finally:
            self.getPage("/exit")
        p.join()



import logging

from sqlalchemy.engine import engine_from_config

try:
    from logging.config import dictConfig
except ImportError:
    from logutils.dictconfig import dictConfig

from cherrypy.process.plugins import SimplePlugin


__all__ = ['LoggingPlugin', 'SQLAlchemyPlugin']


class LoggingPlugin(SimplePlugin):
    """ Sets up process-wide application specific loggers
    """

    def __init__(self, bus,
                 raiseExceptions=logging.raiseExceptions,
                 config={}):

        SimplePlugin.__init__(self, bus)

        self.raiseExceptions = raiseExceptions
        self.config = config

    def start(self):
        try:
            logging.raiseExceptions = self.raiseExceptions
            dictConfig(self.config)
            self.bus.log("Loggers configured")
        except Exception:
            self.bus.log(traceback=True)
    start.priority = 80

    def exit(self):
        self.bus.log("Flushing and closing loggers.")
        logging.shutdown()


class SQLAlchemyPlugin(SimplePlugin):
    """Sets up process-wide SQLAlchemy engines.
    
    This plugin setups basic machinary to attach and clean up engine bindings.
    Engine bindings are in the exact format as the `binds` keyword in
    `Session.configure()`.
    
    In the future in case we ever get to horizontal sharding, this plugin will
    need to be updated.
    """

    def __init__(self, bus, config, prefix="sqlalchemy_engine"):
        SimplePlugin.__init__(self, bus)
        self.config = config
        self.prefix = prefix

    def start(self):
        self._configure_engines()
        self.bus.log("SQLAlchemy Plugin started")
    start.priority = 83

    def graceful(self):
        if hasattr(self, "engine_bindings"):
            engine_bindings = self.engine_bindings
            for engine in engine_bindings.itervalues():
                self.bus.log("Disposing SQLAlchemy engine %s ..." % engine.url)
                engine.dispose()
        elif hasattr(self, "engine"):
            engine = self.engine
            self.bus.log("Disposing SQLAlchemy engine %s ..." % engine.url)
            engine.dispose()
    stop = graceful

    def _configure_engines(self):
        """Sets up engine bindings based on the given config.
        
        Given a configuration dictionary, and optionally a key `prefix`, this
        method iterates all its sections looking for sections with names that
        start with `prefix`. The suffix of the name is fully qualified name to
        a model. A SQLAlchemy engine is then set up with the section value as
        configuration parameters to `sqlalchemy.engine_form_config`. The
        configured engine bindings are to be found in
        `cherrypy.engine.sqlalchemy.engine_bindings` (or wherever you've
        attached this plugin to).
        
        If there is a section that is named exactly the same as the `prefix`,
        that section's values are use to configure only one SQLAlchemy engine
        attached to `cherrypy.engine.sqlalchemy.engine`.
        
        Example::
        
            # The model to be imported starts after the _ following the prefix
            [sqlalchemy_engine_myproject.models.User]
            url = ...
            pool_recycle = ...
            
            # If this section exists, only 1 engine will be configured
            [sqlalchemy_engine]
            url = ...
        
        
        :py:func: sqlalchemy.engine_from_config
        """

        if self.prefix in self.config:
            section = self.config[self.prefix]
            self.engine = engine_from_config(section, '')
            self.bus.log("SQLAlchemy engine configured")
        else:
            engine_bindings = {}

            for section_name, section in self.config.iteritems():
                if section_name.startswith(self.prefix):
                    model_fqn = section_name[len(self.prefix) + 1:]
                    model_fqn_parts = model_fqn.rsplit('.', 1)
                    model_mod = __import__(model_fqn_parts[0], globals(), locals(), [model_fqn_parts[1]])
                    model = getattr(model_mod, model_fqn_parts[1])
                    engine_bindings[model] = engine_from_config(section, '')

            self.engine_bindings = engine_bindings

            self.bus.log("SQLAlchemy engines configured")



import __builtin__
import textwrap
try:
    import unittest2 as unittest
except ImportError:
    import unittest
import warnings
from StringIO import StringIO

from jinja2.loaders import DictLoader
from webassets import Environment

from blueberrypy.config import BlueberryPyConfiguration
from blueberrypy.exc import BlueberryPyConfigurationError, BlueberryPyNotConfiguredError


# dummy controllers
import cherrypy

class Root(object):

    def index(self):
        return "hello world!"
    index.exposed = True

class DummyRestController(object):

    def dummy(self, **kwargs):
        return "hello world!"

rest_controller = cherrypy.dispatch.RoutesDispatcher()
rest_controller.connect("dummy", "/dummy", DummyRestController, action="dummy")


class BlueberryPyConfigurationTest(unittest.TestCase):

    def setUp(self):
        self.basic_valid_app_config = {"controllers": {'': {"controller": Root},
                                                       "/api": {"controller": rest_controller,
                                                                '/': {"request.dispatch": rest_controller}}}}

    def test_validate(self):
        self.assertRaisesRegexp(BlueberryPyNotConfiguredError,
                                "BlueberryPy application configuration not found.",
                                BlueberryPyConfiguration)

    def test_config_file_paths(self):
        # stub out os.path.exists
        import os.path
        old_exists = os.path.exists
        def proxied_exists(path):
            if path == "/tmp/dev/app.yml":
                return True
            elif path == "/tmp/dev/bundles.yml":
                return True
            elif path == "/tmp/dev/logging.yml":
                return True
            return old_exists(path)
        os.path.exists = proxied_exists
        old_open = __builtin__.open

        # stub out open
        class FakeFile(StringIO):
            def __enter__(self):
                return self
            def __exit__(self, exc_type=None, exc_value=None, traceback=None):
                return False
        def proxied_open(filename, mode='r', buffering=1):
            if filename == "/tmp/dev/app.yml":
                return FakeFile()
            elif filename == "/tmp/dev/bundles.yml":
                return FakeFile(textwrap.dedent("""
                directory: /tmp
                url: /
                """))
            elif filename == "/tmp/dev/logging.yml":
                return FakeFile()
            else:
                return old_open(filename, mode, buffering)
        __builtin__.open = proxied_open

        # stub out validate()
        old_validate = BlueberryPyConfiguration.validate
        BlueberryPyConfiguration.validate = lambda self: None

        try:
            config = BlueberryPyConfiguration(config_dir="/tmp")
            config_file_paths = config.config_file_paths
            self.assertEqual(len(config_file_paths), 3)
            self.assertEqual(config_file_paths[0], "/tmp/dev/app.yml")
            self.assertEqual(config_file_paths[1], "/tmp/dev/bundles.yml")
            self.assertEqual(config_file_paths[2], "/tmp/dev/logging.yml")
        finally:
            BlueberryPyConfiguration.validate = old_validate
            os.path.exists = old_exists
            __builtin__.open = old_open

    def test_use_email(self):
        app_config = self.basic_valid_app_config.copy()
        app_config.update({"email": {}})

        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("error")
            self.assertRaisesRegexp(UserWarning,
                                    "BlueberryPy email configuration is empty.",
                                    callable_obj=BlueberryPyConfiguration,
                                    app_config=app_config)

        app_config.update({"email": {"debug": 1}})
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("error")
            self.assertRaisesRegexp(UserWarning,
                                    "Unknown key 'debug' found for \[email\]. Did you mean 'debuglevel'?",
                                    callable_obj=BlueberryPyConfiguration,
                                    app_config=app_config)

        app_config.update({"email": {"host": "localhost",
                                     "port": 1025}})
        config = BlueberryPyConfiguration(app_config=app_config)
        self.assertTrue(config.use_email)

    def test_use_redis(self):
        app_config = self.basic_valid_app_config.copy()
        config = BlueberryPyConfiguration(app_config=app_config)
        self.assertFalse(config.use_redis)

        app_config["controllers"][''].update({"/": {"tools.sessions.storage_type": "redis"}})

        config = BlueberryPyConfiguration(app_config=app_config)
        self.assertTrue(config.use_redis)

    def test_use_sqlalchemy(self):
        app_config = self.basic_valid_app_config.copy()
        app_config.update({"global": {"engine.sqlalchemy.on": True}})

        self.assertRaisesRegexp(BlueberryPyNotConfiguredError,
                                "SQLAlchemy configuration not found.",
                                callable_obj=BlueberryPyConfiguration,
                                app_config=app_config)

        app_config.update({"global": {"engine.sqlalchemy.on": True},
                           "sqlalchemy_engine": {"url": "sqlite://"}})

        config = BlueberryPyConfiguration(app_config=app_config)
        self.assertTrue(config.use_sqlalchemy)

        app_config.update({"global": {"engine.sqlalchemy.on": True},
                           "sqlalchemy_engine_Model": {"url": "sqlite://"}})

        config = BlueberryPyConfiguration(app_config=app_config)
        self.assertTrue(config.use_sqlalchemy)

    def test_use_jinja2(self):
        app_config = self.basic_valid_app_config.copy()
        app_config.update({"jinja2": {}})
        self.assertRaisesRegexp(BlueberryPyNotConfiguredError,
                                "Jinja2 configuration not found.",
                                callable_obj=BlueberryPyConfiguration,
                                app_config=app_config)

        app_config.update({"jinja2": {"loader": DictLoader({})}})
        config = BlueberryPyConfiguration(app_config=app_config)
        self.assertTrue(config.use_jinja2)

    def test_use_webassets(self):
        app_config = self.basic_valid_app_config.copy()
        app_config.update({"jinja2": {"use_webassets": True,
                                      "loader": DictLoader({})}})
        self.assertRaisesRegexp(BlueberryPyNotConfiguredError,
                                "Webassets configuration not found.",
                                callable_obj=BlueberryPyConfiguration,
                                app_config=app_config)

        webassets_env = Environment("/tmp", "/")
        self.assertRaisesRegexp(BlueberryPyNotConfiguredError,
                                "No bundles found in webassets env.",
                                callable_obj=BlueberryPyConfiguration,
                                app_config=app_config,
                                webassets_env=webassets_env)

        webassets_env = Environment("/tmp", "/")
        webassets_env.register("js", "dummy.js", "dummy2.js", output="dummy.js")
        config = BlueberryPyConfiguration(app_config=app_config,
                                          webassets_env=webassets_env)
        self.assertTrue(config.use_webassets)

    def test_jinja2_config(self):
        app_config = self.basic_valid_app_config.copy()
        dict_loader = DictLoader({})
        app_config.update({"jinja2": {"loader": dict_loader,
                                      "use_webassets": True}})

        webassets_env = Environment("/tmp", "/")
        webassets_env.register("js", "dummy.js", "dummy2.js", output="dummy.js")
        config = BlueberryPyConfiguration(app_config=app_config,
                                          webassets_env=webassets_env)
        self.assertEqual(config.jinja2_config, {"loader": dict_loader})

    def test_sqlalchemy_config(self):
        app_config = self.basic_valid_app_config.copy()
        app_config.update({"global": {"engine.sqlalchemy.on": True},
                           "sqlalchemy_engine": {"url": "sqlite://"}})

        config = BlueberryPyConfiguration(app_config=app_config)
        self.assertEqual(config.sqlalchemy_config, {"sqlalchemy_engine": {"url": "sqlite://"}})

        app_config = self.basic_valid_app_config.copy()
        app_config.update({"global": {"engine.sqlalchemy.on": True},
                           "sqlalchemy_engine_Model1": {"url": "sqlite://"},
                           "sqlalchemy_engine_Model2": {"url": "sqlite://"}})

        config = BlueberryPyConfiguration(app_config=app_config)
        self.assertEqual(config.sqlalchemy_config, {"sqlalchemy_engine_Model1": {"url": "sqlite://"},
                                                    "sqlalchemy_engine_Model2": {"url": "sqlite://"}})

    def test_controllers_config(self):
        app_config = {"global": {}}
        self.assertRaisesRegexp(BlueberryPyConfigurationError,
                                "You must declare at least one controller\.",
                                callable_obj=BlueberryPyConfiguration,
                                app_config=app_config)

        app_config = {"controllers": {}}
        self.assertRaisesRegexp(BlueberryPyConfigurationError,
                                "You must declare at least one controller\.",
                                callable_obj=BlueberryPyConfiguration,
                                app_config=app_config)

        app_config = {"controllers": {'api': {'tools.json_in.on': True}}}
        self.assertRaisesRegexp(BlueberryPyConfigurationError,
                                "You must define a controller in the \[controllers\]\[api\] section\.",
                                callable_obj=BlueberryPyConfiguration,
                                app_config=app_config)

        class Root(object):
            def index(self):
                return "hello world!"

        app_config = {"controllers": {"": {"controller": Root}}}
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("error")
            self.assertRaisesRegexp(UserWarning,
                                    "Controller '' has no exposed method\.",
                                    callable_obj=BlueberryPyConfiguration,
                                    app_config=app_config)

        class Root(object):
            def index(self):
                return "hello world!"
            index.exposed = True

        app_config = {"controllers": {"": {"controller": Root}}}
        config = BlueberryPyConfiguration(app_config=app_config)

        rest_controller = cherrypy.dispatch.RoutesDispatcher()

        app_config = {"controllers": {"/api": {"controller": rest_controller}}}
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("error")
            self.assertRaisesRegexp(UserWarning,
                                    "Controller '/api' has no connected routes\.",
                                    callable_obj=BlueberryPyConfiguration,
                                    app_config=app_config)

        class DummyRestController(object):
            def dummy(self, **kwargs):
                return "hello world!"

        rest_controller.connect("dummy", "/dummy", DummyRestController, action="dummy")
        app_config = {"controllers": {"/api": {"controller": rest_controller}}}
        config = BlueberryPyConfiguration(app_config=app_config)



import copy
import hashlib
import hmac
import warnings

from base64 import b64encode, urlsafe_b64encode

try:
    import simplejson as json
except ImportError:
    import json

from datetime import date, time, datetime, timedelta

from dateutil.parser import parse as parse_date
from sqlalchemy.orm import RelationshipProperty, Session, collections

try:
    from geoalchemy.base import SpatialElement, WKTSpatialElement
    from shapely.geometry import asShape, mapping as asGeoJSON
    from shapely.wkb import loads as wkb_decode
    from shapely.wkt import loads as wkt_decode
except ImportError:
    geos_support = False
else:
    geos_support = True


__all__ = ["to_collection", "to_mapping", "from_mapping", "CSRFToken",
           "pad_block_cipher_message", "unpad_block_cipher_message"]


def _get_model_properties(model, excludes):
    props = {}
    for prop in model.__mapper__.iterate_properties:
        if isinstance(prop, RelationshipProperty):
            props[prop.key] = prop
            if prop.backref:
                backref_prop_key = prop.backref[0]
                for mapper in prop.mapper.polymorphic_iterator():
                    excludes.setdefault(mapper.class_, set()).add(backref_prop_key)
        else:
            if prop.key.startswith("_"):
                props[prop.columns[0].key] = prop
            else:
                props[prop.key] = prop
    return props

def _ensure_is_dict(key, inc_exc):

    if inc_exc:
        inc_exc = copy.deepcopy(inc_exc)

        if isinstance(inc_exc, basestring):
            inc_exc = {key: set([inc_exc])}
        elif isinstance(inc_exc, (list, tuple, set, frozenset)):
            inc_exc = {key: set(iter(inc_exc))}
        elif not isinstance(inc_exc, dict):
            raise TypeError(inc_exc, "Please provide a string, an iterable or a dict")

        return inc_exc

    return {}

def to_mapping(value, includes=None, excludes=None, format=None, recursive=False, **json_kwargs):
    warnings.warn("to_mapping() is deprecated and will be removed in 0.6, please use to_collection() instead.")
    return to_collection(value, includes=includes, excludes=excludes, format=format,
                         recursive=recursive, **json_kwargs)

def to_collection(from_, includes=None, excludes=None, format=None, recursive=False, **json_kwargs):
    """Utility function to convert complex values and SQLAlchemy declarative model objects to a Python collection.
    
    This function generally works very similar to `json.dump()`, with the
    following enhancements:
    
    SQLAlchemy declarative model
    ----------------------------
    If `from_` is a SQLAlchemy declarative model object (identified by the
    existance of a `__mapper__` attribute), or a collection if it,
    `to_collection()` will iterate through all the value's mapped properties
    and put the mapped property's name and its value into the result object to
    be returned. In addition to basic Python data types, this function will
    convert `datetime` values according to the following table:

    ========== =========== =============
    value type result key  result value
    ========== =========== =============
    datetime   datetime    .isoformat()
    time       time        .isoformat()
    date       date        .isoformat()
    timedelta  interval    .seconds
    ========== =========== =============
    
    Furthermore, GeoAlchemy `SpatialElement values are also converted to
    `geojson <http://geojson.org/>`_ format using
    `Shapely <http://toblerity.github.com/shapely/>_`.
    
    If `includes` is provided, additional attribute(s) in the model value(s)
    will be included in the returned result. `includes` can be a string, an
    iterable of strings, or a mapping of classes to iterables of strings. This
    is usually used for getting the values of the un-mapped properties from the
    model instances.
    
    If `excludes` is provided, which can also be a string, an iterable of
    strings, or a mapping of classes to iterables of strings, the attribute(s)
    will be excluded from the returned result.
    
    Internally, `to_collection()` will convert the provided `includes` and
    `excludes` property sets to a mapping of the classes of the values to lists
    of property key strings.
    
    **Note:** Mapped property names starting with '_' will never be included in the
    returned result.

    If `recursive` is True, `to_collection` will recursively traverse the entire
    object graph of the values and return a result representing the entire
    object tree. The backrefs of the relationship properties will be
    automatically added to the `excludes` set to prevent running into an
    infinite loop. If you set `recursive` to True, and also supply either an
    `includes` or `excludes` property sets, it is encouraged that you provide
    mappings for explicitness.

    Complex values
    --------------
    If `from_` is not a a SQLAlchemy declarative model, it must be a Python
    collection and its elements are processed according to the same logic as
    SQLAlchemy mode. If `from_` is a collection, this function will recursively
    convert all elements if `recursive` is True. `includes` and `excludes` will
    have no effect under this mode unless some decendent objects are SQLAlchemy
    declarative model objects, in which case processing will be the same as
    described above.
    
    **Note:** If `from_` is an instance of a dict its keys will be converted to
    a string regardless. All iterables besides a dict is returned as a list.
    
    
    If `format` is the string `json`, the result returned will be a JSON string
    , otherwise a Python collection object will be returned.
    
    If any `json_kwargs` is provided, they will be passed through to the
    underlying simplejson JSONDecoder.
    
    Examples:
    ---------
    >>> to_collection(legco) #doctest: +SKIP
    {'name': 'Hong Kong Legislative Council Building', 'founded': {'date': '1912-01-15'}, 'location': {'type': 'Point', 'coordinates': (22.280909, 114.160349)}}
    
    >>> to_collection(legco, excludes=['founded', 'location']) #doctest: +SKIP
    {'name': 'Hong Kong Legislative Council Building'}
    
    >>> to_collection(legco, excludes='founded', format='json') #doctest: +SKIP
    '{"name": "Hong Kong Legislative Council Building", 'location': {'type': 'Point', 'coordinates': [22.280909, 114.160349]}}'
    
    >>> to_collection([legco, hkpark], recursive=True, included={Location: set(['founded'])}) #doctest: +SKIP
    [{'name': 'Hong Kong Legislative Council Building', 'founded': {'date': '1912-01-15'}, 'location': {'type': 'Point', 'coordinates': (22.280909, 114.160349)}},
    {'name': 'Hong Kong Park', 'founded': {'date': '1991-05-23'}, 'location': {'type': 'Point', 'coordinates': [22.2771398, 114.1613993]}}]
    """
    if hasattr(from_, "__mapper__"):

        includes = _ensure_is_dict(from_.__class__, includes)
        excludes = _ensure_is_dict(from_.__class__, excludes)

        props = _get_model_properties(from_, excludes)
        attrs = set(props.iterkeys())
        if includes and from_.__class__ in includes:
            attrs |= includes[from_.__class__]
        if excludes and from_.__class__ in excludes:
            attrs -= excludes[from_.__class__]

        result = {}
        for attr in attrs:
            if not attr.startswith("_"):
                val = getattr(from_, attr)
                val = to_collection(val, includes=includes, excludes=excludes, recursive=recursive)
                result[attr] = val
    else:
        if isinstance(from_, datetime):
            result = {"datetime": from_.isoformat()}
        elif isinstance(from_, time):
            result = {"time": from_.isoformat()}
        elif isinstance(from_, date):
            result = {"date": from_.isoformat()}
        elif isinstance(from_, timedelta):
            result = {"interval": from_.seconds}
        elif geos_support and isinstance(from_, SpatialElement):
            if isinstance(from_, WKTSpatialElement):
                result = asGeoJSON(wkt_decode(from_.geom_wkt))
            else:
                result = asGeoJSON(wkb_decode(str(from_.geom_wkb)))
        elif isinstance(from_, dict):
            result = {}
            for k, v in from_.items():
                result[unicode(k)] = to_collection(v, includes=includes, excludes=excludes, recursive=recursive)
        elif hasattr(from_, "__iter__"): # iterable collections, not strings
            result = [to_collection(v, includes=includes, excludes=excludes, recursive=recursive)
                      for v in from_] if recursive else list(from_)
        else:
            result = from_

    if format == "json":
        return json.dumps(result, **json_kwargs)

    return result

def from_mapping(mapping, instance, excludes=None, format=None):
    warnings.warn("from_mapping() is deprecated and will be removed in 0.6, please use from_collection() instead.")
    return from_collection(mapping, instance, excludes=excludes, format=format)

def _get_property_instance(session, mapping, prop):
    prop_cls = prop.mapper.class_
    prop_pk_vals = tuple((mapping[pk_col.key] for pk_col in prop.mapper.primary_key if pk_col.key in mapping))
    if prop_pk_vals:
        prop_inst = session.query(prop_cls).get(prop_pk_vals)
    elif prop.mapper.polymorphic_on is not None:
        prop_inst = prop.mapper.polymorphic_map[mapping[prop.mapper.get_property_by_column(prop.mapper.polymorphic_on).key]].class_()
    else:
        prop_inst = prop_cls()
    return prop_inst

def from_collection(from_, to_, excludes=None, format=None, collection_handling="replace"):
    """Utility function to recursively apply data in a Python collection to SQLAlchemy declarative model objects.
    
    This function takes a `from_` and an `to_` and sets the attributes on the
    SQLAlchemy declarative model instance using the key-value pairs from the
    collection **inplace**.
    
    If `excludes` is provided, it works similarily as `to_collection`.
    
    If `format` is the string `json`, the mapping returned will be a JSON
    string, otherwise the mapped model(s) will be returned.
    
    If `collection_handling` is `replace`, which is the default, all the
    supplied relationship mappings will be converted to the correct subclass
    instances and replace the entire relationship collection on the parent
    objects. If the value is `append`, the mapped model instance will be
    appended to the relationship collection instead.
    
    If a key from the mapping is not found as a column on a model instance, it
    will simply be skipped and not set on the instance.
    
    The values supplied is converted according to the similiar rules as
    `to_collection()`:
    
    ============== ============================================
    column type    mapping value format
    ============== ============================================
    datetime       {"datetime": "ISO-8601"}
    time           {"time": "ISO-8601"}
    date           {"date": "ISO-8601"}
    timedelta      {"interval": seconds}
    SpatialElement GeoJSON
    ============== ============================================
    
    **Security Notice:** This function currently does not yet have integration 
    support for data validation. If you are using this function to directly 
    mass-assign user supplied data to your model instances, make sure you have 
    validated the data first. In a future version of blueberrypy, integration 
    with a form validation library will be provided to ease this process. 
    """
    if format == "json":
        from_ = json.loads(from_)

    if not isinstance(from_, dict):
        if hasattr(from_, "__iter__"):
            if not hasattr(to_, "__iter__"):
                raise TypeError("to_ must be an iterable if from_ is an iterable.")
            elif len(from_) != len(to_):
                raise ValueError("length of to_ must match length of from_.")

    if collection_handling != "replace" and collection_handling != "append":
        raise ValueError("collection_handling must be 'replace' or 'append'.")

    excludes = _ensure_is_dict(to_.__class__, excludes)

    if isinstance(from_, dict):
        if isinstance(to_, dict):
            for k in to_.iterkeys():
                if k in from_:
                    to_[k] = from_collection(from_[k], to_[k], excludes=excludes)
        elif hasattr(to_, "__mapper__"):
            props = _get_model_properties(to_, excludes)
            attrs = set(props.iterkeys())
            if excludes and to_.__class__ in excludes:
                attrs -= excludes[to_.__class__]

            for attr in attrs:
                if attr in from_:
                    prop = props[attr]
                    from_val = from_[attr]
                    if isinstance(prop, RelationshipProperty):
                        if not isinstance(from_val, list) and not isinstance(from_val, dict):
                            raise ValueError("%r must be either a list or a dict" % attr)

                        if prop.uselist is None or prop.uselist:

                            if collection_handling == "replace":
                                col = collections.prepare_instrumentation(prop.collection_class or list)()
                            elif collection_handling == "append":
                                col = getattr(to_, attr)

                            appender = col._sa_appender

                            from_iterator = iter(from_val) if isinstance(from_val, list) else from_val.itervalues()

                            for v in from_iterator:
                                prop_inst = _get_property_instance(Session.object_session(to_), v, prop)
                                appender(from_collection(v, prop_inst, excludes=excludes))
                            
                            if collection_handling == "replace":
                                setattr(to_, attr, col)
                        else:
                            prop_inst = _get_property_instance(Session.object_session(to_), from_val, prop)
                            setattr(to_, attr, from_collection(from_val, prop_inst, excludes=excludes))
                    else:
                        setattr(to_, attr, from_collection(from_val, None, excludes=excludes))
        else:
            if "date" in from_:
                to_ = parse_date(from_["date"]).date()
            elif "time" in from_:
                to_ = parse_date(from_["time"]).time()
            elif "datetime" in from_:
                to_ = parse_date(from_["datetime"])
            elif "interval" in from_:
                to_ = timedelta(seconds=from_["interval"])
            elif geos_support and "type" in from_:
                to_ = WKTSpatialElement(asShape(from_).wkt)

    elif hasattr(from_, "__iter__") and hasattr(to_, "__iter__"):
        to_ = [from_collection(f, t, excludes=excludes) for f, t in zip(from_, to_)]

    else:
        to_ = from_

    return to_


class CSRFToken(object):

    def __init__(self, path, secret, session_id, urlsafe=False):
        self.path = str(path)
        self.secret = str(secret)
        self.session_id = str(session_id)
        self.urlsafe = urlsafe
        self.token = self.generate(urlsafe)

    def generate(self, urlsafe=False):
        mac = hmac.new(self.secret, digestmod=hashlib.sha256)
        mac.update(self.path)
        mac.update(self.session_id)

        if urlsafe or self.urlsafe:
            self.token = urlsafe_b64encode(mac.digest())
        else:
            self.token = b64encode(mac.digest())

        return self.token

    def verify(self, other):
        return str(self) == str(other)

    def __str__(self):
        return self.token

    def __repr__(self):
        return "CSRFToken(%r, %r, %r, %r)" % (self.path, self.secret, self.session_id, self.urlsafe)


def pad_block_cipher_message(msg, block_size=16, padding='{'):
    return msg + (block_size - len(msg) % block_size) * padding

def unpad_block_cipher_message(msg, padding="{"):
    return msg.rstrip(padding)



from __future__ import absolute_import

import logging
import smtplib
import socket
import time
import warnings

from email.header import Header
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.utils import parseaddr, formataddr


logger = logging.getLogger(__name__)


class Mailer(object):

    def __init__(self, host='', port=0, local_hostname=None,
                 timeout=socket._GLOBAL_DEFAULT_TIMEOUT,
                 ssl=False, keyfile=None, certfile=None,
                 default_sender=None, debuglevel=False, connection_retries=10):

        self.host = host
        self.port = port
        self.local_hostname = local_hostname
        self.timeout = timeout
        self.ssl = ssl
        self.keyfile = keyfile
        self.certfile = certfile
        self.default_sender = default_sender
        self.debuglevel = debuglevel
        self.connection_retries = connection_retries

    def _get_connection(self):
        if self.ssl:
            connection = smtplib.SMTP_SSL(self.host, self.port,
                                          self.local_hostname,
                                          self.keyfile, self.certfile,
                                          self.timeout)
        else:
            connection = smtplib.SMTP(self.host, self.port, self.local_hostname,
                                      self.timeout)
        connection.set_debuglevel(self.debuglevel)
        return connection

    def send_email(self, to_, from_=None, subject=None, body=None,
                  subtype="plain", charset="utf-8"):

        message = MIMEText(body, subtype, charset)

        if subject:
            subject_header = Header()
            subject = unicode(subject, charset) if isinstance(subject, str) else subject
            subject_header.append(subject.strip())
            message["Subject"] = subject_header

        from_ = from_ or self.default_sender
        from_ = unicode(from_, charset) if isinstance(from_, str) else from_
        from_realname, from_addr = parseaddr(from_)
        from_header = Header()
        from_header.append(formataddr((from_realname, from_addr)))
        message['From'] = from_header

        to_ = unicode(to_, charset) if isinstance(to_, str) else to_
        to_realname, to_addr = parseaddr(to_)
        to_header = Header()
        to_header.append(formataddr((to_realname, to_addr)))
        message['To'] = to_header

        self._send(message, from_addr, to_addr)

    def _send(self, mime_message, from_, to_):
        connection = self._get_connection()
        try:
            connection.sendmail(from_, to_, mime_message.as_string(False))
        except smtplib.SMTPHeloError, e:
            logger.error(e, exc_info=True)
            raise
        except smtplib.SMTPSenderRefused, e:
            logger.error(e, exc_info=True)
            raise
        except smtplib.SMTPDataError, e:
            logger.error(e, exc_info=True)
            raise
        except smtplib.SMTPServerDisconnected, e:
            tries = 0
            exp_timeout = 2 ** tries
            exception = None
            while tries < self.connection_retries:
                try:
                    logger.warn("Server disconnected, retrying in %s seconds...", exp_timeout)
                    time.sleep(exp_timeout)
                    connection.sendmail(from_, to_, mime_message.as_string(False))
                    break
                except smtplib.SMTPException, e:
                    tries = tries + 1
                    exp_timeout = 2 ** tries
                    exception = e
            else:
                logger.error(exception, exc_info=True)
                raise exception
        finally:
            connection.quit()

    def send_html_email(self, to_, from_=None, subject=None, text=None,
                        html=None, charset="utf-8"):

        message = MIMEMultipart("alternative")

        if subject:
            subject_header = Header()
            subject = unicode(subject, charset) if isinstance(subject, str) else subject
            subject_header.append(subject.strip())
            message["Subject"] = subject_header

        from_ = from_ or self.default_sender
        from_ = unicode(from_, charset) if isinstance(from_, str) else from_
        from_realname, from_addr = parseaddr(from_)
        from_header = Header()
        from_header.append(formataddr((from_realname, from_addr)))
        message['From'] = from_header

        to_ = unicode(to_, charset) if isinstance(to_, str) else to_
        to_realname, to_addr = parseaddr(to_)
        to_header = Header()
        to_header.append(formataddr((to_realname, to_addr)))
        message['To'] = to_header

        message.attach(MIMEText(text, "plain", charset))
        message.attach(MIMEText(html, "html", charset))

        self._send(message, from_addr, to_addr)


_mailer = None

def configure(email_config):
    global _mailer
    _mailer = Mailer(**email_config)

def send_email(to_, from_=None, subject=None, body=None, subtype="plain",
               charset="utf-8"):

    if _mailer is None:
        warnings.warn("Module %s not configured." % __name__)
    else:
        return _mailer.send_email(to_, from_, subject, body, subtype, charset)

def send_html_email(to_, from_=None, subject=None, text=None, html=None,
                    charset="utf-8"):

    if _mailer is None:
        warnings.warn("Module %s not configured." % __name__)
    else:
        return _mailer.send_html_email(to_, from_, subject, text, html, charset)



from __future__ import print_function

import sys
try:
    import cdecimal
    sys.modules["decimal"] = cdecimal
except:
    pass

import argparse
import logging
import os
import re
import textwrap

from datetime import datetime

import cherrypy
from cherrypy.process import servers
from cherrypy.process.plugins import Daemonizer, DropPrivileges, PIDFile

import blueberrypy
from blueberrypy.config import BlueberryPyConfiguration
from blueberrypy.project import create_project
from blueberrypy.console import Console
from blueberrypy.template_engine import configure_jinja2
from blueberrypy.exc import BlueberryPyNotConfiguredError


def create(args, config_dir=None):

    path = None
    if args.path:
        path = os.path.abspath(args.path)
        if not os.path.exists(path):
            os.mkdir(path)
            print("Path not found, a directory '%s' has been created." % path, file=sys.stderr)

    valid_version_re = re.compile(r"^\d+.\d+(.\d+)*(a|b|c|rc\d+(.\d+)?)?(.post\d+)?(.dev\d+)?$")
    valid_email_re = re.compile(r"^.+@.+$")

    blueberrypy_config = {}
    blueberrypy_config["path"] = path or os.getcwdu()
    blueberrypy_config["current_year"] = datetime.now().year

    blueberrypy_config["project_name"] = raw_input("Project name: ")

    while True:
        blueberrypy_config["package"] = package = raw_input("Package name: ")
        if not package:
            print("Package name cannot be empty.", file=sys.stderr)
        else:
            break

    while True:
        blueberrypy_config["version"] = version = raw_input("Version (PEP 386): ")
        if not valid_version_re.match(version):
            print("'%s' does not match the required version format." % version, file=sys.stderr)
        else:
            break

    blueberrypy_config["author"] = raw_input("Author name: ")

    while True:
        blueberrypy_config["email"] = email = raw_input("Email: ")
        if not valid_email_re.match(email):
            print("'%s' is an invalid email address." % email, file=sys.stderr)
        else:
            break

    while True:
        use_controller = raw_input("Use controllers backed by a templating engine? [Y/n] ").lower()
        if use_controller and (not use_controller.startswith('y') and not use_controller.startswith('n')):
            print("Please answer Y or N.", file=sys.stderr)
        else:
            blueberrypy_config["use_controller"] = True if not use_controller or use_controller[0] == 'y' else False
            break

    while True:
        use_rest_controller = raw_input("Use RESTful controllers? [y/N] ").lower()
        if use_rest_controller and (not use_rest_controller.startswith('y') and not use_rest_controller.startswith('n')):
            print("Please answer Y or N.", file=sys.stderr)
        else:
            blueberrypy_config["use_rest_controller"] = False if not use_rest_controller or use_rest_controller[0] == 'n' else True
            break

    if blueberrypy_config["use_controller"]:
        while True:
            use_jinja2 = raw_input("Use Jinja2 templating engine? [Y/n] ").lower()
            if use_jinja2 and (not use_jinja2.startswith('y') and not use_jinja2.startswith('n')):
                print("Please answer Y or N.", file=sys.stderr)
            else:
                blueberrypy_config["use_jinja2"] = True if not use_jinja2 or use_jinja2[0] == 'y' else False
                break

        while True:
            use_webassets = raw_input("Use webassets asset management framework? [Y/n] ").lower()
            if use_webassets and (not use_webassets.startswith('y') and not use_webassets.startswith('n')):
                print("Please answer Y or N.", file=sys.stderr)
            else:
                blueberrypy_config["use_webassets"] = True if not use_webassets or use_webassets[0] == 'y' else False
                break
    else:
        blueberrypy_config["use_jinja2"] = False
        blueberrypy_config["use_webassets"] = False

    while True:
        use_redis = raw_input("Use redis session? [y/N] ").lower()
        if use_redis and (not use_redis.startswith('y') and not use_redis.startswith('n')):
            print("Please answer Y or N.", file=sys.stderr)
        else:
            blueberrypy_config["use_redis"] = False if not use_redis or use_redis[0] == 'n' else True
            break

    while True:
        use_sqlalchemy = raw_input("Use SQLAlchemy ORM? [Y/n] ").lower()
        if use_sqlalchemy and (not use_sqlalchemy.startswith('y') and not use_sqlalchemy.startswith('n')):
            print("Please answer Y or N.", file=sys.stderr)
        else:
            blueberrypy_config["use_sqlalchemy"] = True if not use_sqlalchemy or use_sqlalchemy[0] == 'y' else False
            if blueberrypy_config["use_sqlalchemy"]:
                blueberrypy_config["sqlalchemy_url"] = sqlalchemy_url = raw_input("SQLAlchemy database connection URL: ")
                if sqlalchemy_url.strip():
                    try:
                        from sqlalchemy.engine import url as sa_url
                        blueberrypy_config["driver"] = sa_url.make_url(sqlalchemy_url).get_dialect().driver
                    except ImportError, e:
                        pass
                break
            break

    create_project(blueberrypy_config, dry_run=args.dry_run)

    footer = textwrap.dedent("""
    ===========================================================================
    Your project skeleton has been created under {path}.
 
    Subsystems chosen
    -----------------
    Routes (RESTful controllers): {use_rest_controller}
    Jinja2: {use_jinja2}
    webassets: {use_webassets}
    redis: {use_redis}
    SQLAlchemy: {use_sqlalchemy}
    
    If you now install your package now the packages above will be automatically
    installed as well.

    e.g. $ pip install -e .
 
    In unrestricted environments, you may also install 'MarkupSafe' and
    'cdecimal' to speed up Jinja2 and SQLAlchemy's queries on Decimal fields
    respectively. You may also install 'hiredis' if you have opted for the Redis
    session storage.

    e.g. $ pip install blueberrypy[speedups]
    
    You should also install the appropriate database driver if you have decided
    to use BlueberryPy's SQLAlchemy support.

    For more information, the BlueberryPy documentation is available at
    http://blueberrypy.readthedocs.org.

    Happy coding!
    """.format(**blueberrypy_config))

    print(footer)

def bundle(args, config_dir=None):

    config = BlueberryPyConfiguration(config_dir=config_dir)

    assets_env = config.webassets_env
    if not assets_env:
        raise BlueberryPyNotConfiguredError("Webassets configuration not found.")

    logging.basicConfig()
    logger = logging.getLogger(__name__)

    from webassets.script import CommandLineEnvironment
    assets_cli = CommandLineEnvironment(assets_env, logger)

    if args.build:
        try:
            assets_cli.build()
        except AttributeError:
            assets_cli.rebuild()
    elif args.watch:
        assets_cli.watch()
    elif args.clean:
        assets_cli.clean()

def serve(args, config_dir=None):

    config = BlueberryPyConfiguration(config_dir=config_dir)

    cpengine = cherrypy.engine

    cpenviron = args.environment
    if cpenviron:
        config = BlueberryPyConfiguration(config_dir=config_dir,
                                          environment=cpenviron)
        cherrypy.config.update({"environment": cpenviron})

    if config.use_email and config.email_config:
        from blueberrypy import email
        email.configure(config.email_config)

    if config.use_logging and config.logging_config:
        from blueberrypy.plugins import LoggingPlugin
        cpengine.logging = LoggingPlugin(cpengine, config=config.logging_config)

    if config.use_redis:
        from blueberrypy.session import RedisSession
        cherrypy.lib.sessions.RedisSession = RedisSession

    if config.use_sqlalchemy:
        from blueberrypy.plugins import SQLAlchemyPlugin
        cpengine.sqlalchemy = SQLAlchemyPlugin(cpengine,
                                               config=config.sqlalchemy_config)
        from blueberrypy.tools import SQLAlchemySessionTool
        cherrypy.tools.orm_session = SQLAlchemySessionTool()

    if config.use_jinja2:
        if config.webassets_env:
            configure_jinja2(assets_env=config.webassets_env,
                             **config.jinja2_config)
        else:
            configure_jinja2(**config.jinja2_config)

    # update global config first, so subsequent command line options can
    # override the settings in the config files
    cherrypy.config.update(config.app_config)

    if args.binding:
        address, port = args.binding.strip().split(":")
        cherrypy.server.socket_host = address
        cherrypy.server.socket_port = int(port)

    if args.daemonize:
        cherrypy.config.update({'log.screen': False})
        Daemonizer(cpengine).subscribe()

    if args.drop_privilege:
        cherrypy.config.update({'engine.autoreload_on': False})
        DropPrivileges(cpengine, umask=args.umask or 022,
                       uid=args.uid or "www",
                       gid=args.gid or "www").subscribe()

    if args.pidfile:
        PIDFile(cpengine, args.pidfile).subscribe()

    fastcgi, scgi = getattr(args, 'fastcgi'), getattr(args, 'scgi')
    if fastcgi and scgi:
        cherrypy.log.error("You may only specify one of the fastcgi and "
                           "scgi options.", 'ENGINE')
        sys.exit(1)
    elif fastcgi or scgi:
        # Turn off autoreload when using *cgi.
        cherrypy.config.update({'engine.autoreload_on': False})
        # Turn off the default HTTP server (which is subscribed by default).
        cherrypy.server.unsubscribe()

        addr = cherrypy.server.bind_addr
        if fastcgi:
            f = servers.FlupFCGIServer(application=cherrypy.tree,
                                       bindAddress=addr)
        elif scgi:
            f = servers.FlupSCGIServer(application=cherrypy.tree,
                                       bindAddress=addr)
        s = servers.ServerAdapter(cpengine, httpserver=f, bind_addr=addr)
        s.subscribe()

    if hasattr(cpengine, 'signal_handler'):
        cpengine.signal_handler.subscribe()

    # for win32 only
    if hasattr(cpengine, "console_control_handler"):
        cpengine.console_control_handler.subscribe()

    # mount the controllers
    for script_name, section in config.controllers_config.iteritems():
        section = section.copy()
        controller = section.pop("controller")
        if isinstance(controller, cherrypy.dispatch.RoutesDispatcher):
            routes_config = {'/': {"request.dispatch": controller}}
            for path in section.iterkeys():
                if path.strip() == '/':
                    routes_config['/'].update(section['/'])
                else:
                    routes_config[path] = section[path].copy()
            app_config = config.app_config.copy()
            app_config.pop("controllers")
            routes_config.update(app_config)
            cherrypy.tree.mount(None, script_name=script_name,
                                config=routes_config)
        else:
            app_config = config.app_config.copy()
            app_config.pop("controllers")
            controller_config = section.copy()
            controller_config.update(app_config)
            cherrypy.tree.mount(controller(), script_name=script_name,
                                config=controller_config)

    # Add the blueberrypy config files into CP's autoreload monitor
    # Jinja2 templates are monitored by Jinja2 itself and will autoreload if
    # needed
    if config.config_file_paths:
        for path in config.config_file_paths:
            cpengine.autoreload.files.add(path)

    try:
        cpengine.start()
    except:
        sys.exit(1)
    else:
        cpengine.block()

def console(args, config_dir=None):
    banner = """
*****************************************************************************
* If the configuration file you specified contains a [sqlalchemy_engine*]   *
* section, a default SQLAlchemy engine and session should have been created *
* for you automatically already.                                            *
*****************************************************************************
"""
    cherrypy.config.update({"environment": args.environment})
    Console(BlueberryPyConfiguration(config_dir=config_dir,
                                     environment=args.environment)).interact(banner)

def create_parser(config_dir=None):
    parser = argparse.ArgumentParser(prog="create",
                                     description="Create a project skeleton.")
    parser.add_argument("-p", "--path", help="the path to create the project. "
                        "default is the current directory")
    parser.add_argument("-d", "--dry-run", action="store_true", default=False,
                         help="do not write skeleton. prints out the content instead")
    return parser

def bundle_parser(config_dir=None):

    description = textwrap.dedent("""Webassets bundle management.
    
    Before you can use this command to bundle up your Web assets, you should
    have created either a project skeleton using the 'create' command or
    provided a configuration directory using the global option -c --config_dir.
    """)

    config = BlueberryPyConfiguration(config_dir=config_dir)
    if not config.use_webassets:
        print(description)
        sys.exit(1)

    parser = argparse.ArgumentParser(prog="bundle", description=description)
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument("-b", "--build", action="store_true", dest="build",
                        help="build the asset bundles")

    group.add_argument("-w", "--watch", action="store_true", dest="watch",
                        help="automatically rebuild the asset bundles upon changes in the static directory")

    group.add_argument("-c", "--clean", action="store_true", dest="clean",
                        help="delete the generated asset bundles")
    return parser

def serve_parser(config_dir=None):

    parser = argparse.ArgumentParser(prog="serve",
                                     description="spawn a new running cherrypy process")

    parser.add_argument("-b", "--bind", dest="binding",
                         default="127.0.0.1:8080",
                         help="the address and port to bind to. [default: %(default)s]")
    parser.add_argument("-e", "--environment", dest="environment",
                         default=None,
                         help="apply the given config environment")
    parser.add_argument("-f", action="store_true", dest="fastcgi",
                         help="start a fastcgi server instead of the default HTTP server")
    parser.add_argument("-s", action="store_true", dest="scgi",
                         help="start a scgi server instead of the default HTTP server")
    parser.add_argument("-d", "--daemonize", action="store_true",
                         default=False,
                         help="run the server as a daemon. [default: %(default)s]")
    parser.add_argument("-p", "--drop-privilege", action="store_true",
                         default=False,
                         help="drop privilege to separately specified umask, "
                         "uid and gid. [default: %(default)s]")
    parser.add_argument('-P', '--pidfile', dest='pidfile', default=None,
                         help="store the process id in the given file")
    parser.add_argument("-u", "--uid", default="www",
                         help="setuid to uid [default: %(default)s]")
    parser.add_argument("-g", "--gid", default="www",
                         help="setgid to gid [default: %(default)s]")
    parser.add_argument("-m", "--umask", default="022", type=int,
                         help="set umask [default: %(default)s]")
    return parser

def console_parser(config_dir=None):

    parser = argparse.ArgumentParser(prog="bundle",
                                     description="An REPL fully configured for experimentation.")
    parser.add_argument("-e", "--environment", dest="environment",
                         default=None,
                         help="apply the given config environment")

    return parser

def main():
    description = textwrap.dedent("""BlueberryPy lightweight pluggable Web application framework command line interface.
    
    Type 'blueberrypy -h' or 'blueberrypy --help' for general help.
    Type 'blueberrypy help <command>' for help on that specific command.
    
    commands:
    
    help                print this help or a command's if an argument is given
    create              create a project skeleton
    console             BlueberryPy REPL for experimentations
    bundle              bundles up web assets (type 'blueberrypy help bundle' for details)
    serve               spawn a new CherryPy server process
    """)

    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=description)
    parser.add_argument("-v", "--version", action="store_true", default=False,
                        help="print version information and exit.")

    parser.add_argument("-C", "--config_dir", help="path to the config directory")

    parser.add_argument("command", nargs="?", default="help", help="the action to perform")

    args, extraargs = parser.parse_known_args()
    if args.version:
        print("BlueberryPy version %s" % blueberrypy.__version__)
        parser.exit(0)

    def get_command_parser_and_callback(command):
        if command == "create":
            return (create_parser(args.config_dir), create)
        elif command == "console":
            return (console_parser(args.config_dir), console)
        elif command == "bundle":
            return (bundle_parser(args.config_dir), bundle)
        elif command == "serve":
            return (serve_parser(args.config_dir), serve)

        parser.error("Unknown command %r" % args.command)

    if args.command == "help":
        if extraargs:
            subparser, _ = get_command_parser_and_callback(extraargs[0])
            subparser.print_help()
        else:
            parser.print_help()
    else:
        subparser, func = get_command_parser_and_callback(args.command)
        cmdargs = subparser.parse_args(extraargs)
        func(cmdargs, args.config_dir)



import functools
import hashlib
import hmac

try:
    import unittest2 as unittest
except ImportError:
    import unittest

from base64 import b64encode
from datetime import date, time, datetime, timedelta

import testconfig

from geoalchemy import GeometryColumn, Point, WKTSpatialElement, GeometryDDL
from sqlalchemy import Column, Integer, Date, DateTime, Time, Interval, Enum, \
    ForeignKey, UnicodeText, engine_from_config
from sqlalchemy.orm import sessionmaker, scoped_session, relationship, backref
from sqlalchemy.ext.declarative import declarative_base

from blueberrypy.util import CSRFToken, pad_block_cipher_message, \
    unpad_block_cipher_message, from_collection, to_collection


engine = engine_from_config(testconfig.config["sqlalchemy_engine"], '')
Session = scoped_session(sessionmaker(engine))

Base = declarative_base()
metadata = Base.metadata
metadata.bind = engine


class RelatedEntity(Base):

    __tablename__ = "related"

    id = Column(Integer, autoincrement=True, primary_key=True)
    key = Column(UnicodeText)

    parent_id = Column(Integer, ForeignKey("testentity.id"))

    discriminator = Column("type", Enum("related", "relatedsubclass",
                                        name="searchoptiontype"))

    __mapper_args__ = {"polymorphic_on": discriminator,
                       "polymorphic_identity": "related"}


class RelatedEntitySubclass(RelatedEntity):

    __mapper_args__ = {"polymorphic_identity": "relatedsubclass"}

    subclass_prop = Column(UnicodeText)


# remember to setup postgis
class TestEntity(Base):

    __tablename__ = "testentity"

    discriminator = Column("type", Enum("base", "derived", name="entitytype"),
                           nullable=False)

    __mapper_args__ = {"polymorphic_on": discriminator,
                       "polymorphic_identity": "base"}

    id = Column(Integer, primary_key=True)
    date = Column(Date)
    time = Column(Time)
    datetime = Column(DateTime)
    interval = Column(Interval)
    geo = GeometryColumn(Point(2))

    @property
    def combined(self):
        return datetime.combine(self.date, self.time)

    related = relationship(RelatedEntity, backref=backref("parent"))


GeometryDDL(TestEntity.__table__)


class DerivedTestEntity(TestEntity):

    __tablename__ = "derivedtestentity"

    __mapper_args__ = {"polymorphic_identity": "derived"}

    id = Column(Integer, ForeignKey("testentity.id"), nullable=False,
                primary_key=True)

    derivedprop = Column(Integer)


def orm_session(func):
    def _orm_session(*args, **kwargs):
        session = Session()
        try:
            return func(*args, **kwargs)
        except:
            raise
        finally:
            session.close()
    return functools.update_wrapper(_orm_session, func)


class CSRFTokenTest(unittest.TestCase):

    def test_csrftoken(self):
        csrftoken = CSRFToken("/test", "secret", 1)

        mac = hmac.new("secret", digestmod=hashlib.sha256)
        mac.update("/test")
        mac.update('1')
        testtoken = b64encode(mac.digest())

        self.assertEqual(str(csrftoken), testtoken)
        self.assertTrue(csrftoken.verify(testtoken))

        mac = hmac.new("secret2", digestmod=hashlib.sha256)
        mac.update("/test")
        mac.update('1')
        testtoken = b64encode(mac.digest())

        self.assertNotEqual(str(csrftoken), testtoken)
        self.assertFalse(csrftoken.verify(testtoken))

        mac = hmac.new("secret", digestmod=hashlib.sha256)
        mac.update("/test2")
        mac.update('1')
        testtoken = b64encode(mac.digest())

        self.assertNotEqual(str(csrftoken), testtoken)
        self.assertFalse(csrftoken.verify(testtoken))

        mac = hmac.new("secret", digestmod=hashlib.sha256)
        mac.update("/test2")
        mac.update('2')
        testtoken = b64encode(mac.digest())

        self.assertNotEqual(str(csrftoken), testtoken)
        self.assertFalse(csrftoken.verify(testtoken))


class CollectionUtilTest(unittest.TestCase):

    @classmethod
    @orm_session
    def setup_class(cls):
        metadata.create_all(engine)

        te = DerivedTestEntity(id=1,
                               date=date(2012, 1, 1),
                               time=time(0, 0, 0),
                               derivedprop=2,
                               datetime=datetime(2012, 1, 1, 0, 0, 0),
                               interval=timedelta(seconds=3600),
                               geo=WKTSpatialElement("POINT(45.0 45.0)"))
        session = Session()
        session.add(te)

        te.related = [RelatedEntity(key=u"related1"),
                      RelatedEntitySubclass(key=u"related2", subclass_prop=u"sub1")]

        session.commit()

        te2 = TestEntity(id=2,
                         date=date(2013, 2, 2),
                         time=time(1, 1, 1),
                         datetime=datetime(2013, 2, 2, 1, 1, 1),
                         interval=timedelta(seconds=3601),
                         geo=WKTSpatialElement("POINT(46.0 45.1)"))

        session = Session()
        session.add(te2)

        te2.related = [RelatedEntity(key=u"related3"),
                      RelatedEntity(key=u"related4")]

        session.commit()
    setUpClass = setup_class

    @classmethod
    @orm_session
    def teardown_class(cls):
        metadata.drop_all(engine)
    testDownClass = teardown_class

    @orm_session
    def test_to_collection(self):

        self.assertEqual(1, to_collection(1))
        self.assertEqual(1.1, to_collection(1.1))
        self.assertEqual("str", to_collection("str"))
        self.assertEqual([1, 2, 3], to_collection([1, 2, 3]))
        self.assertEqual([1, 2, 3], to_collection((1, 2, 3)))
        self.assertEqual([1, 2, 3], to_collection(set([1, 2, 3])))
        self.assertEqual([1, 2, 3], to_collection(frozenset([1, 2, 3])))
        self.assertEqual({"1": [2]}, to_collection({1: [2]}))
        self.assertEqual({"a": [1, 2], "b": 2}, to_collection({"a": set([1, 2]), "b": 2}))

        doc = {'date': {'date': '2012-01-01'},
               'time': {'time': '00:00:00'},
               'interval': {'interval': 3600},
               'id': 1,
               'discriminator': 'derived',
               'derivedprop': 2,
               'datetime': {'datetime': '2012-01-01T00:00:00'},
               'geo': {'type': 'Point',
                       'coordinates': (45.0, 45.0)},
               'related': [{'id': 1,
                            'discriminator': 'related',
                            'key': u'related1',
                            'parent_id': 1},
                           {'id': 2,
                            'discriminator': 'relatedsubclass',
                            'key': u'related2',
                            'parent_id': 1,
                            'subclass_prop': u'sub1'}]}

        session = Session()
        te = session.query(TestEntity).get(1)
        result = to_collection(te, recursive=True)

        self.assertEqual(doc, result)

        serialized_doc = '{"date": {"date": "2012-01-01"}, "datetime": {"datetime": "2012-01-01T00:00:00"}, "derivedprop": 2, "discriminator": "derived", "geo": {"coordinates": [45.0, 45.0], "type": "Point"}, "id": 1, "interval": {"interval": 3600}, "related": [{"discriminator": "related", "id": 1, "key": "related1", "parent_id": 1}, {"discriminator": "relatedsubclass", "id": 2, "key": "related2", "parent_id": 1, "subclass_prop": "sub1"}], "time": {"time": "00:00:00"}}'
        result = to_collection(te, format="json", recursive=True, sort_keys=True)
        self.assertEqual(serialized_doc, result)

        doc = {'date': {'date': '2012-01-01'},
               'time': {'time': '00:00:00'},
               'discriminator': 'derived',
               'datetime': {'datetime': '2012-01-01T00:00:00'},
               'combined': {'datetime': '2012-01-01T00:00:00'},
               'geo': {'type': 'Point', 'coordinates': (45.0, 45.0)}}

        self.assertEqual(doc, to_collection(te, includes=["combined"],
                                            excludes=["id", "interval", "derivedprop", "related"]))
        self.assertEqual("a", to_collection("a"))
        self.assertEqual(1, to_collection(1))
        self.assertEqual(1.1, to_collection(1.1))
        self.assertEqual({'date': '2012-01-01'}, to_collection(date(2012, 1, 1)))
        self.assertEqual({'time': '00:00:00'}, to_collection(time(0, 0, 0)))
        self.assertEqual({'interval': 3600}, to_collection(timedelta(seconds=3600)))
        self.assertEqual({'datetime': '2012-01-01T00:00:00'}, to_collection(datetime(2012, 1, 1, 0, 0, 0)))
        self.assertEqual({'type': 'Point', 'coordinates': (45.0, 45.0)}, to_collection(te.geo))

        tes = session.query(TestEntity).all()
        result = to_collection(tes, recursive=True,
                               includes={DerivedTestEntity: set(['combined'])},
                               excludes={DerivedTestEntity: set(['id', 'interval', 'derivedprop'])},
                               format="json", sort_keys=True)

        serialized_doc = '[{"combined": {"datetime": "2012-01-01T00:00:00"}, "date": {"date": "2012-01-01"}, "datetime": {"datetime": "2012-01-01T00:00:00"}, "discriminator": "derived", "geo": {"coordinates": [45.0, 45.0], "type": "Point"}, "related": [{"discriminator": "related", "id": 1, "key": "related1", "parent_id": 1}, {"discriminator": "relatedsubclass", "id": 2, "key": "related2", "parent_id": 1, "subclass_prop": "sub1"}], "time": {"time": "00:00:00"}}, {"combined": {"datetime": "2013-02-02T01:01:01"}, "date": {"date": "2013-02-02"}, "datetime": {"datetime": "2013-02-02T01:01:01"}, "discriminator": "derived", "geo": {"coordinates": [46.0, 45.1], "type": "Point"}, "related": [{"discriminator": "related", "id": 3, "key": "related3", "parent_id": 2}, {"discriminator": "related", "id": 4, "key": "related4", "parent_id": 2}], "time": {"time": "01:01:01"}}]'
        self.assertEqual(serialized_doc, result)

    @orm_session
    def test_from_collection(self):

        self.assertEqual(1, from_collection(1, None))
        self.assertEqual(1.1, from_collection(1.1, None))
        self.assertEqual("str", from_collection("str", None))
        self.assertEqual([1, 2, 3], from_collection([1, 2, 3], [4, 5, 6]))
        self.assertEqual([1, 2, 3], from_collection((1, 2, 3), (4, 5, 6)))
        self.assertEqual([1, 2, 3], from_collection(set([1, 2, 3]), set([4, 5, 6])))
        self.assertEqual([1, 2, 3], from_collection(frozenset([1, 2, 3]), frozenset([4, 5, 6])))

        doc = {'date': {'date': '2012-01-01'},
               'time': {'time': '00:00:00'},
               'interval': {'interval': 3600},
               'id': 1,
               'derivedprop': 2,
               'datetime': {'datetime': '2012-01-01T00:00:00'},
               'geo': {'type': 'Point', 'coordinates': (45.0, 45.0)},
               'related': [{'key': u'key1', 'parent_id': 1, 'discriminator': 'related'},
                           {'key': u'key2', 'parent_id': 1, 'discriminator': 'relatedsubclass', 'subclass_prop': 'sub'}]}
        
        te = DerivedTestEntity()
        te = from_collection(doc, te)
        self.assertEqual(te.date, date(2012, 1, 1))
        self.assertEqual(te.time, time(0, 0, 0))
        self.assertEqual(te.interval, timedelta(seconds=3600))
        self.assertEqual(te.datetime, datetime(2012, 1, 1, 0, 0, 0))
        self.assertEqual(te.id, 1)
        self.assertEqual(te.derivedprop, 2)
        self.assertEqual(te.geo.geom_wkt, "POINT (45.0000000000000000 45.0000000000000000)")
        self.assertIsNone(te.related[0].id)
        self.assertEqual(te.related[0].parent_id, 1)
        self.assertEqual(te.related[0].key, "key1")
        self.assertEqual(te.related[0].discriminator, "related")
        self.assertIsNone(te.related[1].id)
        self.assertEqual(te.related[1].parent_id, 1)
        self.assertEqual(te.related[1].key, "key2")
        self.assertEqual(te.related[1].discriminator, "relatedsubclass")
        self.assertEqual(te.related[1].subclass_prop, "sub")

        #TODO: testing loading of persisted entity, json format, excludes
        doc = {'date': {'date': '2012-01-01'},
               'time': {'time': '00:00:00'},
               'interval': {'interval': 3600},
               'id': 1,
               'datetime': {'datetime': '2012-01-01T00:00:00'},
               'geo': {'type': 'Point', 'coordinates': (45.0, 45.0)},
               'related': [{'key': u'key1', 'parent_id': 1, 'discriminator': u'related', "id": 3}]}

        session = Session()
        te = session.query(TestEntity).get(2)
        te = from_collection(doc, te, excludes=["interval"])
        self.assertEqual(te.date, date(2012, 1, 1))
        self.assertEqual(te.time, time(0, 0, 0))
        self.assertEqual(te.interval, timedelta(seconds=3601))
        self.assertEqual(te.datetime, datetime(2012, 1, 1, 0, 0, 0))
        self.assertEqual(te.id, 1)
        self.assertEqual(te.geo.geom_wkt, "POINT (45.0000000000000000 45.0000000000000000)")
        self.assertEqual(te.related[0].parent_id, 1)
        self.assertEqual(te.related[0].key, u"key1")
        self.assertEqual(te.related[0].id, 3)
        self.assertEqual(te.related[0].discriminator, u"related")
        self.assertEqual(len(te.related), 1)
        self.assertIsNotNone(Session.object_session(te.related[0]))
        
        doc = {'related': [{'key': u'hello', 'parent_id': 1, 'discriminator': u'related'}]}
        te = from_collection(doc, te, collection_handling="append")
        self.assertEqual(len(te.related), 2)
        self.assertEqual(te.related[-1].key, u"hello")
        self.assertEqual(te.related[-1].parent_id, 1)
        self.assertEqual(te.related[-1].discriminator, "related")

        te = DerivedTestEntity()
        json_doc = '{"time": {"time": "00:00:00"}, "date": {"date": "2012-01-01"}, "geo": {"type": "Point", "coordinates": [45.0, 45.0]}, "interval": {"interval": 3600}, "datetime": {"datetime": "2012-01-01T00:00:00"}, "id": 1, "related": [{"parent_id": 1, "key": "key1", "discriminator": "related"}, {"parent_id": 1, "subclass_prop": "sub", "key": "key2", "discriminator": "relatedsubclass"}], "derivedprop": 2}'
        te = from_collection(json_doc, te, format="json")
        self.assertEqual(te.date, date(2012, 1, 1))
        self.assertEqual(te.time, time(0, 0, 0))
        self.assertEqual(te.interval, timedelta(seconds=3600))
        self.assertEqual(te.datetime, datetime(2012, 1, 1, 0, 0, 0))
        self.assertEqual(te.id, 1)
        self.assertEqual(te.derivedprop, 2)
        self.assertEqual(te.geo.geom_wkt, "POINT (45.0000000000000000 45.0000000000000000)")
        self.assertIsNone(te.related[0].id)
        self.assertEqual(te.related[0].parent_id, 1)
        self.assertEqual(te.related[0].key, "key1")
        self.assertEqual(te.related[0].discriminator, "related")
        self.assertIsNone(te.related[1].id)
        self.assertEqual(te.related[1].parent_id, 1)
        self.assertEqual(te.related[1].key, "key2")
        self.assertEqual(te.related[1].discriminator, "relatedsubclass")
        self.assertEqual(te.related[1].subclass_prop, "sub")


class BlockCipherPaddingTest(unittest.TestCase):

    def test_pad_block_cipher_message(self):
        padded_message = pad_block_cipher_message("message")
        self.assertEqual(padded_message, "message{{{{{{{{{")

    def test_unpad_block_cipher_message(self):
        self.assertEqual(unpad_block_cipher_message("message{{{{{{{{{"), "message")



from __future__ import print_function

import difflib
import os.path
import re
import shutil

from cherrypy.test.webtest import getchar
from jinja2 import Environment as Jinja2Environment


def project_template_filter(blueberrypy_config, path):

    if path.endswith(".html_tmpl"):
        return Jinja2Environment(block_start_string="<%",
                                 block_end_string="%>",
                                 variable_start_string="<<",
                                 variable_end_string=">>",
                                 comment_start_string="<#",
                                 comment_end_string="#>")

    if path.endswith("_tmpl"):
        path = path[:-5]

    if path.endswith("rest_controller.py"):
        return blueberrypy_config.get("use_rest_controller")
    if path.endswith("controller.py") or path.find("static") != -1:
        return blueberrypy_config.get("use_controller")
    if path.find("/templates") != -1:
        return blueberrypy_config.get("use_controller") and \
            blueberrypy_config.get("use_jinja2")
    if path.endswith("bundles.yml"):
        return blueberrypy_config.get("use_webassets")
    if path.endswith(".hidden"):
        return False
    if path.endswith("model.py") or path.endswith("api.py"):
        return blueberrypy_config.get("use_sqlalchemy")

    return True


class ProjectCreator(object):

    def __init__(self, blueberrypy_config, dry_run=False,
                 project_template_filter=project_template_filter):

        self.dry_run = dry_run
        self.filename_var_re = re.compile(r"\+(.+)\+")
        self.blueberrypy_config = blueberrypy_config.copy()
        self.dest = blueberrypy_config["path"]
        self.project_template_dir = os.path.join(os.path.dirname(__file__), "project_template")
        self.overwrite = False

        self.process_dir(self.project_template_dir)

    def replace_path(self, path):
        matcher = self.filename_var_re.search(path)
        while matcher:
            var = matcher.group(1)
            val = self.blueberrypy_config[var]
            start, end = matcher.span(1)
            path = path[:start - 1] + val + path[end + 1:]
            matcher = self.filename_var_re.search(path)
        return path

    def process_dir(self, root, directory=''):

        currentpath = os.path.join(root, directory)

        for listing in os.listdir(currentpath):

            relpath = os.path.join(directory, listing)
            src_fullpath = os.path.join(currentpath, listing)
            replaced_relpath = self.replace_path(relpath)

            proceed = project_template_filter(self.blueberrypy_config, relpath)

            # hack to preprocess jinja2 html templates
            if isinstance(proceed, Jinja2Environment):
                jinja2_env = proceed
            else:
                jinja2_env = Jinja2Environment()

            if proceed:

                if os.path.isdir(src_fullpath):
                    dest_fullpath = os.path.join(self.dest, replaced_relpath)

                    if not self.dry_run:
                        if not os.path.exists(dest_fullpath):
                            os.makedirs(dest_fullpath)
                    else:
                        print("created directory " + dest_fullpath)

                    self.process_dir(root, relpath)
                else:
                    dest_fullpath = os.path.join(self.dest, replaced_relpath)

                    head, tail = os.path.split(dest_fullpath)

                    if tail.endswith("_tmpl"):
                        dest_fullpath = os.path.join(head, tail[:-5])

                        with open(src_fullpath) as infile:
                            tmpl = jinja2_env.from_string(infile.read())

                        if not self.dry_run:
                            if self.overwrite or not os.path.exists(dest_fullpath):
                                with open(dest_fullpath, 'w') as outfile:
                                    outfile.write(tmpl.render(self.blueberrypy_config))
                            elif os.path.exists(dest_fullpath):
                                while True:
                                    print("%s already exists, do you wish to overwrite it?" % dest_fullpath,
                                    "[Y]es [N]o [A]ll [C]ompare the files >> ", sep='\n', end='')
                                    i = getchar().upper()

                                    if not isinstance(i, type("")):
                                        i = i.decode('ascii')

                                    if i not in "YNAC":
                                        continue

                                    print(i.upper())

                                    if i == 'Y' or i == 'A':
                                        with open(dest_fullpath, 'w') as outfile:
                                            outfile.write(tmpl.render(self.blueberrypy_config))

                                        if i == 'A':
                                            self.overwrite = True

                                        break

                                    elif i == 'N':
                                        print("%s has been skipped." % dest_fullpath)
                                        break

                                    elif i == 'C':
                                        with open(dest_fullpath) as infile:
                                            old = infile.read()
                                        new = tmpl.render(self.blueberrypy_config)
                                        print("\n".join(difflib.unified_diff(old.splitlines(), new.splitlines(), dest_fullpath + ".old", dest_fullpath + ".new")))
                                        continue
                        else:
                            print("created file " + dest_fullpath)
                            print(tmpl.render(self.blueberrypy_config))
                    else:
                        if not self.dry_run:
                            if self.overwrite or not os.path.exists(dest_fullpath):
                                shutil.copy(src_fullpath, dest_fullpath)
                            elif os.path.exists(dest_fullpath):
                                while True:
                                    print("\n%s already exists, do you wish to overwrite it?" % dest_fullpath,
                                    "[Y]es [N]o [A]ll [C]ompare the files >> ", sep='\n', end='')
                                    i = getchar().upper()

                                    if not isinstance(i, type("")):
                                        i = i.decode('ascii')

                                    if i not in "YNAC":
                                        continue

                                    print(i.upper())

                                    if i == 'Y' or i == 'A':
                                        shutil.copy(src_fullpath, dest_fullpath)

                                        if i == 'A':
                                            self.overwrite = True

                                        break

                                    elif i == 'N':
                                        print("%s has been skipped." % dest_fullpath)
                                        break

                                    elif i == 'C':
                                        with open(dest_fullpath) as infile:
                                            old = infile.read()
                                        with open(src_fullpath) as infile:
                                            new = infile.read()
                                        print("\n".join(difflib.unified_diff(old.splitlines(), new.splitlines(), dest_fullpath + ".old", dest_fullpath + ".new")))
                                        continue
                        else:
                            print("copied file " + dest_fullpath)

create_project = ProjectCreator



import __builtin__
import os.path
import sys
try:
    import unittest2 as unittest
except ImportError:
    import unittest
import textwrap

from StringIO import StringIO

import cherrypy
import jinja2
import webassets

from yaml import load as load_yaml

from ludibrio import Mock

import blueberrypy
from blueberrypy.command import main


class CreateCommandTest(unittest.TestCase):

    def test_create_command_y_with_path(self):
        sys.argv = "blueberrypy create -p /tmp".split()

        with Mock() as raw_input:
            from __builtin__ import raw_input
            raw_input("Project name: ") >> "testproject"
            raw_input("Package name: ") >> "testproject"
            raw_input("Version (PEP 386): ") >> "0.1"
            raw_input("Author name: ") >> "author"
            raw_input("Email: ") >> "alice@example.com"
            raw_input("Use controllers backed by a templating engine? [Y/n] ") >> "y"
            raw_input("Use RESTful controllers? [y/N] ") >> "y"
            raw_input("Use Jinja2 templating engine? [Y/n] ") >> "y"
            raw_input("Use webassets asset management framework? [Y/n] ") >> "y"
            raw_input("Use redis session? [y/N] ") >> "y"
            raw_input("Use SQLAlchemy ORM? [Y/n] ") >> "y"
            raw_input("SQLAlchemy database connection URL: ") >> "sqlite://"

        config = {'author': 'author',
                  'current_year': 2012,
                  'driver': 'pysqlite',
                  'email': 'alice@example.com',
                  'package': 'testproject',
                  'path': '/tmp',
                  'project_name': 'testproject',
                  'sqlalchemy_url': 'sqlite://',
                  'use_controller': True,
                  'use_jinja2': True,
                  'use_redis': True,
                  'use_rest_controller': True,
                  'use_sqlalchemy': True,
                  'use_webassets': True,
                  'version': '0.1'}

        with Mock() as create_project:
            from blueberrypy.command import create_project
            create_project(config, dry_run=False) >> None

        main()

        raw_input.validate()
        create_project.validate()

    def test_create_command_yes(self):
        sys.argv = "blueberrypy create".split()

        with Mock() as raw_input:
            from __builtin__ import raw_input
            raw_input("Project name: ") >> "testproject"
            raw_input("Package name: ") >> "testproject"
            raw_input("Version (PEP 386): ") >> "0.1"
            raw_input("Author name: ") >> "author"
            raw_input("Email: ") >> "alice@example.com"
            raw_input("Use controllers backed by a templating engine? [Y/n] ") >> "Yes"
            raw_input("Use RESTful controllers? [y/N] ") >> "Yes"
            raw_input("Use Jinja2 templating engine? [Y/n] ") >> "Yes"
            raw_input("Use webassets asset management framework? [Y/n] ") >> "Yes"
            raw_input("Use redis session? [y/N] ") >> "Yes"
            raw_input("Use SQLAlchemy ORM? [Y/n] ") >> "Yes"
            raw_input("SQLAlchemy database connection URL: ") >> "sqlite://"

        config = {'author': 'author',
                  'current_year': 2012,
                  'driver': 'pysqlite',
                  'email': 'alice@example.com',
                  'package': 'testproject',
                  'path': os.getcwdu(),
                  'project_name': 'testproject',
                  'sqlalchemy_url': 'sqlite://',
                  'use_controller': True,
                  'use_jinja2': True,
                  'use_redis': True,
                  'use_rest_controller': True,
                  'use_sqlalchemy': True,
                  'use_webassets': True,
                  'version': '0.1'}

        with Mock() as create_project:
            from blueberrypy.command import create_project
            create_project(config, dry_run=False) >> None

        main()

        raw_input.validate()
        create_project.validate()

    def test_create_command_n(self):
        sys.argv = "blueberrypy create".split()

        with Mock() as raw_input:
            from __builtin__ import raw_input
            raw_input("Project name: ") >> "testproject"
            raw_input("Package name: ") >> "testproject"
            raw_input("Version (PEP 386): ") >> "0.1"
            raw_input("Author name: ") >> "author"
            raw_input("Email: ") >> "alice@example.com"
            raw_input("Use controllers backed by a templating engine? [Y/n] ") >> "n"
            raw_input("Use RESTful controllers? [y/N] ") >> "n"
            raw_input("Use redis session? [y/N] ") >> "n"
            raw_input("Use SQLAlchemy ORM? [Y/n] ") >> "n"

        config = {'author': 'author',
                  'current_year': 2012,
                  'email': 'alice@example.com',
                  'package': 'testproject',
                  'path': os.getcwdu(),
                  'project_name': 'testproject',
                  'use_controller': False,
                  'use_jinja2': False,
                  'use_redis': False,
                  'use_rest_controller': False,
                  'use_sqlalchemy': False,
                  'use_webassets': False,
                  'version': '0.1'}

        with Mock() as create_project:
            from blueberrypy.command import create_project
            create_project(config, dry_run=False) >> None

        main()

        raw_input.validate()
        create_project.validate()

    def test_create_command_no(self):
        sys.argv = "blueberrypy create".split()

        with Mock() as raw_input:
            from __builtin__ import raw_input
            raw_input("Project name: ") >> "testproject"
            raw_input("Package name: ") >> "testproject"
            raw_input("Version (PEP 386): ") >> "0.1"
            raw_input("Author name: ") >> "author"
            raw_input("Email: ") >> "alice@example.com"
            raw_input("Use controllers backed by a templating engine? [Y/n] ") >> "No"
            raw_input("Use RESTful controllers? [y/N] ") >> "No"
            raw_input("Use redis session? [y/N] ") >> "No"
            raw_input("Use SQLAlchemy ORM? [Y/n] ") >> "No"

        config = {'author': 'author',
                  'current_year': 2012,
                  'email': 'alice@example.com',
                  'package': 'testproject',
                  'path': os.getcwdu(),
                  'project_name': 'testproject',
                  'use_controller': False,
                  'use_jinja2': False,
                  'use_redis': False,
                  'use_rest_controller': False,
                  'use_sqlalchemy': False,
                  'use_webassets': False,
                  'version': '0.1'}

        with Mock() as create_project:
            from blueberrypy.command import create_project
            create_project(config, dry_run=False) >> None

        main()

        raw_input.validate()
        create_project.validate()

    def test_create_command_conditional_no(self):
        sys.argv = "blueberrypy create".split()

        with Mock() as raw_input:
            from __builtin__ import raw_input
            raw_input("Project name: ") >> "testproject"
            raw_input("Package name: ") >> "testproject"
            raw_input("Version (PEP 386): ") >> "0.1"
            raw_input("Author name: ") >> "author"
            raw_input("Email: ") >> "alice@example.com"
            raw_input("Use controllers backed by a templating engine? [Y/n] ") >> "y"
            raw_input("Use RESTful controllers? [y/N] ") >> "No"
            raw_input("Use Jinja2 templating engine? [Y/n] ") >> "No"
            raw_input("Use webassets asset management framework? [Y/n] ") >> "No"
            raw_input("Use redis session? [y/N] ") >> "No"
            raw_input("Use SQLAlchemy ORM? [Y/n] ") >> "No"

        config = {'author': 'author',
                  'current_year': 2012,
                  'email': 'alice@example.com',
                  'package': 'testproject',
                  'path': os.getcwdu(),
                  'project_name': 'testproject',
                  'use_controller': True,
                  'use_jinja2': False,
                  'use_redis': False,
                  'use_rest_controller': False,
                  'use_sqlalchemy': False,
                  'use_webassets': False,
                  'version': '0.1'}

        with Mock() as create_project:
            from blueberrypy.command import create_project
            create_project(config, dry_run=False) >> None

        main()

        raw_input.validate()
        create_project.validate()

    def test_create_command_conditional_n(self):
        sys.argv = "blueberrypy create".split()

        with Mock() as raw_input:
            from __builtin__ import raw_input
            raw_input("Project name: ") >> "testproject"
            raw_input("Package name: ") >> "testproject"
            raw_input("Version (PEP 386): ") >> "0.1"
            raw_input("Author name: ") >> "author"
            raw_input("Email: ") >> "alice@example.com"
            raw_input("Use controllers backed by a templating engine? [Y/n] ") >> "y"
            raw_input("Use RESTful controllers? [y/N] ") >> "No"
            raw_input("Use Jinja2 templating engine? [Y/n] ") >> "n"
            raw_input("Use webassets asset management framework? [Y/n] ") >> "n"
            raw_input("Use redis session? [y/N] ") >> "No"
            raw_input("Use SQLAlchemy ORM? [Y/n] ") >> "No"

        config = {'author': 'author',
                  'current_year': 2012,
                  'email': 'alice@example.com',
                  'package': 'testproject',
                  'path': os.getcwdu(),
                  'project_name': 'testproject',
                  'use_controller': True,
                  'use_jinja2': False,
                  'use_redis': False,
                  'use_rest_controller': False,
                  'use_sqlalchemy': False,
                  'use_webassets': False,
                  'version': '0.1'}

        with Mock() as create_project:
            from blueberrypy.command import create_project
            create_project(config, dry_run=False) >> None

        main()

        raw_input.validate()
        create_project.validate()

    def test_create_command_invalid_input_and_defaults(self):
        sys.argv = "blueberrypy create".split()

        with Mock() as raw_input:
            from __builtin__ import raw_input
            raw_input("Project name: ") >> ""
            raw_input("Package name: ") >> ""
            raw_input("Package name: ") >> "valid_package_name"
            raw_input("Version (PEP 386): ") >> "IWillFail"
            raw_input("Version (PEP 386): ") >> "0.1"
            raw_input("Author name: ") >> ""
            raw_input("Email: ") >> "notavalidemail"
            raw_input("Email: ") >> "alice@example.com"
            raw_input("Use controllers backed by a templating engine? [Y/n] ") >> ""
            raw_input("Use RESTful controllers? [y/N] ") >> ""
            raw_input("Use Jinja2 templating engine? [Y/n] ") >> ""
            raw_input("Use webassets asset management framework? [Y/n] ") >> ""
            raw_input("Use redis session? [y/N] ") >> ""
            raw_input("Use SQLAlchemy ORM? [Y/n] ") >> ""
            raw_input("SQLAlchemy database connection URL: ") >> ""

        config = {'author': '',
                  'current_year': 2012,
                  'email': 'alice@example.com',
                  'package': 'valid_package_name',
                  'path': os.getcwdu(),
                  'project_name': '',
                  'sqlalchemy_url': '',
                  'use_controller': True,
                  'use_jinja2': True,
                  'use_redis': False,
                  'use_rest_controller': False,
                  'use_sqlalchemy': True,
                  'use_webassets': True,
                  'version': '0.1'}

        with Mock() as create_project:
            from blueberrypy.command import create_project
            create_project(config, dry_run=False) >> None

        main()

        raw_input.validate()
        create_project.validate()

    def test_create_command_invalid_yes_no(self):
        sys.argv = "blueberrypy create".split()

        with Mock() as raw_input:
            from __builtin__ import raw_input
            raw_input("Project name: ") >> ""
            raw_input("Package name: ") >> "valid_package_name"
            raw_input("Version (PEP 386): ") >> "0.1"
            raw_input("Author name: ") >> ""
            raw_input("Email: ") >> "alice@example.com"
            raw_input("Use controllers backed by a templating engine? [Y/n] ") >> "sdf"
            raw_input("Use controllers backed by a templating engine? [Y/n] ") >> "y"
            raw_input("Use RESTful controllers? [y/N] ") >> "asdfasdf"
            raw_input("Use RESTful controllers? [y/N] ") >> "y"
            raw_input("Use Jinja2 templating engine? [Y/n] ") >> "asdf"
            raw_input("Use Jinja2 templating engine? [Y/n] ") >> "y"
            raw_input("Use webassets asset management framework? [Y/n] ") >> "asdf"
            raw_input("Use webassets asset management framework? [Y/n] ") >> "y"
            raw_input("Use redis session? [y/N] ") >> "asdfasdf"
            raw_input("Use redis session? [y/N] ") >> "y"
            raw_input("Use SQLAlchemy ORM? [Y/n] ") >> "asdfasdf"
            raw_input("Use SQLAlchemy ORM? [Y/n] ") >> "y"
            raw_input("SQLAlchemy database connection URL: ") >> ""

        config = {'author': '',
                  'current_year': 2012,
                  'email': 'alice@example.com',
                  'package': 'valid_package_name',
                  'path': os.getcwdu(),
                  'project_name': '',
                  'sqlalchemy_url': '',
                  'use_controller': True,
                  'use_jinja2': True,
                  'use_redis': True,
                  'use_rest_controller': True,
                  'use_sqlalchemy': True,
                  'use_webassets': True,
                  'version': '0.1'}

        with Mock() as create_project:
            from blueberrypy.command import create_project
            create_project(config, dry_run=False) >> None

        main()

        raw_input.validate()
        create_project.validate()


# dummy controllers
class Root(object):

    def index(self):
        return "hello world!"
    index.exposed = True

class DummyRestController(object):

    def dummy(self, **kwargs):
        return "hello world!"

rest_controller = cherrypy.dispatch.RoutesDispatcher()
rest_controller.connect("dummy", "/dummy", DummyRestController, action="dummy")


class FakeFile(StringIO):
    def __enter__(self):
        return self
    def __exit__(self, exc_type=None, exc_value=None, traceback=None):
        return False


class ServeCommandTest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.old_sys_argv = sys.argv
        cls.old_cherrypy_server_bind_addr = cherrypy.server.bind_addr

    def setUp(self):
        self.old_exists = os.path.exists
        self.old_open = __builtin__.open

        # mock out engine.block()
        self.old_cherrypy_engine_block = cherrypy.engine.block
        def dummy_block(self):
            pass
        cherrypy.engine.block = dummy_block.__get__(cherrypy.engine, cherrypy.engine.__class__)

    def tearDown(self):
        # clean up attached objects
        if hasattr(cherrypy.engine, "sqlalchemy"):
            del cherrypy.engine.sqlalchemy
        if hasattr(cherrypy.engine, "logging"):
            del cherrypy.engine.logging
        if blueberrypy.email._mailer is not None:
            blueberrypy.email._mailer = None
        if hasattr(cherrypy.lib.sessions, "RedisSession"):
            del cherrypy.lib.sessions.RedisSession
        if hasattr(cherrypy.tools, "orm_session"):
            del cherrypy.tools.orm_session
        if blueberrypy.template_engine.jinja2_env is not None:
            blueberrypy.template_engine.jinja2_env = None

        # restore stuff
        sys.argv = self.old_sys_argv
        cherrypy.server.bind_addr = self.old_cherrypy_server_bind_addr
        os.path.exists = self.old_exists
        __builtin__.open = self.old_open
        cherrypy.engine.block = self.old_cherrypy_engine_block
        cherrypy.engine.exit()

    def _stub_out_path_and_open(self, path_file_mapping=None, mode='r', buffering=1):

        def proxied_exists(path):
            if path in path_file_mapping:
                return True
            return self.old_exists(path)
        os.path.exists = proxied_exists

        if path_file_mapping is None:
            path_file_mapping = {}

        def proxied_open(filename, mode=mode, buffering=buffering):
            if filename in path_file_mapping:
                return path_file_mapping[filename]
            else:
                return self.old_open(filename, mode, buffering)

        __builtin__.open = proxied_open

    def _setup_basic_app_config(self):
        app_yml_file = FakeFile(textwrap.dedent("""
        global:
            environment: test_suite
        controllers:
            '':
                controller: !!python/name:tests.test_command.Root
            /api:
                controller: !!python/name:tests.test_command.rest_controller
        """))

        path_file_mapping = {"/tmp/dev/app.yml": app_yml_file}
        self._stub_out_path_and_open(path_file_mapping)

    def test_setup_email(self):
        app_yml_file = FakeFile(textwrap.dedent("""
        global:
            environment: test_suite
        controllers:
            '':
                controller: !!python/name:tests.test_command.Root
            /api:
                controller: !!python/name:tests.test_command.rest_controller
        email:
            host: localhost
            port: 1025
        """))

        path_file_mapping = {"/tmp/dev/app.yml": app_yml_file}
        self._stub_out_path_and_open(path_file_mapping)

        sys.argv = ("blueberrypy -C /tmp serve").split()
        main()
        self.assertIsInstance(blueberrypy.email._mailer, blueberrypy.email.Mailer)
        self.assertTrue(not hasattr(cherrypy.engine, "logging"))
        self.assertTrue(not hasattr(cherrypy.lib.sessions, "RedisSession"))
        self.assertTrue(not hasattr(cherrypy.engine, "sqlalchemy"))
        self.assertTrue(not hasattr(cherrypy.tools, "orm_session"))
        self.assertIsNone(blueberrypy.template_engine.jinja2_env)

    def test_setup_cherrypy_logging(self):
        app_yml_file = FakeFile(textwrap.dedent("""
        global:
            engine.logging.on: true
            environment: test_suite
        controllers:
            '':
                controller: !!python/name:tests.test_command.Root
            /api:
                controller: !!python/name:tests.test_command.rest_controller
        """))
        logging_yml_file = FakeFile(textwrap.dedent("""
        version: 1
        disable_existing_loggers: false
        loggers:
            root:
                level: WARNING
                handlers: [stdout]
        handlers:
            stdout:
                class: logging.StreamHandler
                formatter: debug
                stream: ext://sys.stdout
        formatters:
            debug:
                format: '%(asctime)s [%(levelname)s] %(name)s Thread(id=%(thread)d, name="%(threadName)s") %(message)s'
        """))

        path_file_mapping = {"/tmp/dev/app.yml": app_yml_file,
                             "/tmp/dev/logging.yml": logging_yml_file}
        self._stub_out_path_and_open(path_file_mapping)

        sys.argv = ("blueberrypy -C /tmp serve").split()
        main()
        self.assertIsNone(blueberrypy.email._mailer)
        self.assertIsInstance(cherrypy.engine.logging, blueberrypy.plugins.LoggingPlugin)
        self.assertTrue(not hasattr(cherrypy.lib.sessions, "RedisSession"))
        self.assertTrue(not hasattr(cherrypy.engine, "sqlalchemy"))
        self.assertTrue(not hasattr(cherrypy.tools, "orm_session"))
        self.assertIsNone(blueberrypy.template_engine.jinja2_env)

    def test_setup_cherrypy_redis(self):
        app_yml_file = FakeFile(textwrap.dedent("""
        global:
            environment: test_suite
        controllers:
            '':
                controller: !!python/name:tests.test_command.Root
                /:
                    tools.sessions.storage_type: redis
            /api:
                controller: !!python/name:tests.test_command.rest_controller
        """))

        path_file_mapping = {"/tmp/dev/app.yml": app_yml_file}
        self._stub_out_path_and_open(path_file_mapping)

        sys.argv = ("blueberrypy -C /tmp serve").split()
        main()
        self.assertIsNone(blueberrypy.email._mailer)
        self.assertTrue(not hasattr(cherrypy.engine, "logging"))
        self.assertEqual(cherrypy.lib.sessions.RedisSession, blueberrypy.session.RedisSession)
        self.assertTrue(not hasattr(cherrypy.engine, "sqlalchemy"))
        self.assertTrue(not hasattr(cherrypy.tools, "orm_session"))
        self.assertIsNone(blueberrypy.template_engine.jinja2_env)

    def test_setup_sqlalchemy(self):
        app_yml_file = FakeFile(textwrap.dedent("""
        global:
            engine.sqlalchemy.on: true
            environment: test_suite
        controllers:
            '':
                controller: !!python/name:tests.test_command.Root
            /api:
                controller: !!python/name:tests.test_command.rest_controller
        sqlalchemy_engine:
            url: sqlite://
        """))

        path_file_mapping = {"/tmp/dev/app.yml": app_yml_file}
        self._stub_out_path_and_open(path_file_mapping)

        sys.argv = ("blueberrypy -C /tmp serve").split()
        main()
        self.assertIsNone(blueberrypy.email._mailer)
        self.assertTrue(not hasattr(cherrypy.engine, "logging"))
        self.assertTrue(not hasattr(cherrypy.lib.sessions, "RedisSession"))
        self.assertIsInstance(cherrypy.engine.sqlalchemy, blueberrypy.plugins.SQLAlchemyPlugin)
        self.assertIsInstance(cherrypy.tools.orm_session, blueberrypy.tools.SQLAlchemySessionTool)
        self.assertIsNone(blueberrypy.template_engine.jinja2_env)

    def test_setup_jinja2(self):
        app_yml_file = FakeFile(textwrap.dedent("""
        global:
            environment: test_suite
        controllers:
            '':
                controller: !!python/name:tests.test_command.Root
            /api:
                controller: !!python/name:tests.test_command.rest_controller
        jinja2:
            loader: !!python/object:jinja2.loaders.DictLoader
                    mapping: {}
        """))

        path_file_mapping = {"/tmp/dev/app.yml": app_yml_file}
        self._stub_out_path_and_open(path_file_mapping)

        sys.argv = ("blueberrypy -C /tmp serve").split()
        main()
        self.assertIsNone(blueberrypy.email._mailer)
        self.assertTrue(not hasattr(cherrypy.engine, "logging"))
        self.assertTrue(not hasattr(cherrypy.lib.sessions, "RedisSession"))
        self.assertTrue(not hasattr(cherrypy.engine, "sqlalchemy"))
        self.assertTrue(not hasattr(cherrypy.tools, "orm_session"))
        self.assertIsInstance(blueberrypy.template_engine.jinja2_env, jinja2.Environment)

    def test_setup_webassets(self):
        app_yml_file = FakeFile(textwrap.dedent("""
        global:
            environment: test_suite
        controllers:
            '':
                controller: !!python/name:tests.test_command.Root
            /api:
                controller: !!python/name:tests.test_command.rest_controller
        jinja2:
            loader: !!python/object:jinja2.loaders.DictLoader
                    mapping: {}
            use_webassets: true
        """))
        bundles_yml_file = FakeFile(textwrap.dedent("""
        directory: /tmp
        url: /
        bundles:
            nosuchjs.js:
                output: nosuchjs.js
                contents: nosuchjs.js
        """))
        path_file_mapping = {"/tmp/dev/app.yml": app_yml_file,
                             "/tmp/dev/bundles.yml": bundles_yml_file}
        self._stub_out_path_and_open(path_file_mapping)

        sys.argv = ("blueberrypy -C /tmp serve").split()
        main()
        self.assertIsNone(blueberrypy.email._mailer)
        self.assertTrue(not hasattr(cherrypy.engine, "logging"))
        self.assertTrue(not hasattr(cherrypy.lib.sessions, "RedisSession"))
        self.assertTrue(not hasattr(cherrypy.engine, "sqlalchemy"))
        self.assertTrue(not hasattr(cherrypy.tools, "orm_session"))
        self.assertIsInstance(blueberrypy.template_engine.jinja2_env, jinja2.Environment)
        self.assertIsInstance(blueberrypy.template_engine.jinja2_env.assets_environment, webassets.Environment)

    def test_setup_cherrypy_all(self):
        app_yml_file = FakeFile(textwrap.dedent("""
        global:
            engine.logging.on: true
            engine.sqlalchemy.on: true
            environment: test_suite
        controllers:
            '':
                controller: !!python/name:tests.test_command.Root
                /:
                    tools.sessions.storage_type: redis
            /api:
                controller: !!python/name:tests.test_command.rest_controller
        sqlalchemy_engine:
            url: sqlite://
        jinja2:
            loader: !!python/object:jinja2.loaders.DictLoader
                    mapping: {}
            use_webassets: true
        email:
            host: localhost
            port: 1025
        """))
        bundles_yml_file = FakeFile(textwrap.dedent("""
        directory: /tmp
        url: /
        bundles:
            nosuchjs.js:
                output: nosuchjs.js
                contents: nosuchjs.js
        """))
        logging_yml_file = FakeFile(textwrap.dedent("""
        version: 1
        disable_existing_loggers: false
        loggers:
            root:
                level: WARNING
                handlers: [stdout]
        handlers:
            stdout:
                class: logging.StreamHandler
                formatter: debug
                stream: ext://sys.stdout
        formatters:
            debug:
                format: '%(asctime)s [%(levelname)s] %(name)s Thread(id=%(thread)d, name="%(threadName)s") %(message)s'
        """))

        path_file_mapping = {"/tmp/dev/app.yml": app_yml_file,
                             "/tmp/dev/bundles.yml": bundles_yml_file,
                             "/tmp/dev/logging.yml": logging_yml_file}
        self._stub_out_path_and_open(path_file_mapping)

        sys.argv = ("blueberrypy -C /tmp serve").split()
        main()
        self.assertIsInstance(blueberrypy.email._mailer, blueberrypy.email.Mailer)
        self.assertIsInstance(cherrypy.engine.logging, blueberrypy.plugins.LoggingPlugin)
        self.assertEqual(cherrypy.lib.sessions.RedisSession, blueberrypy.session.RedisSession)
        self.assertIsInstance(cherrypy.engine.sqlalchemy, blueberrypy.plugins.SQLAlchemyPlugin)
        self.assertIsInstance(cherrypy.tools.orm_session, blueberrypy.tools.SQLAlchemySessionTool)
        self.assertIsInstance(blueberrypy.template_engine.jinja2_env, jinja2.Environment)
        self.assertIsInstance(blueberrypy.template_engine.jinja2_env.assets_environment, webassets.Environment)

    def test_setup_controller(self):
        app_yml_file = FakeFile(textwrap.dedent("""
        global:
            engine.sqlalchemy.on: true
            environment: test_suite
        controllers:
            '':
                controller: !!python/name:tests.test_command.Root
                /:
                    tools.sessions.storage_type: redis
                    tools.orm_session.on: true
        sqlalchemy_engine:
            url: sqlite://
        email:
            host: localhost
            port: 1025
        """))
        path_file_mapping = {"/tmp/dev/app.yml": app_yml_file}
        self._stub_out_path_and_open(path_file_mapping)

        sys.argv = ("blueberrypy -C /tmp serve").split()
        main()

        app_config = load_yaml(app_yml_file.getvalue())
        controller_config = app_config["controllers"][''].copy()
        controller_config.pop("controller")

        merged_app_config = cherrypy.tree.apps[""].config
        for k, v in controller_config.iteritems():
            self.assertEqual(v, merged_app_config[k])

        for k, v in app_config["global"].iteritems():
            self.assertEqual(v, merged_app_config["global"][k])

        for k, v in app_config["email"].iteritems():
            self.assertEqual(v, merged_app_config["email"][k])

        for k, v in app_config["sqlalchemy_engine"].iteritems():
            self.assertEqual(v, merged_app_config["sqlalchemy_engine"][k])

    def test_setup_rest_controller(self):
        app_yml_file = FakeFile(textwrap.dedent("""
        global:
            engine.sqlalchemy.on: true
            environment: test_suite
        controllers:
            /api:
                controller: !!python/name:tests.test_command.rest_controller
                /:
                    tools.sessions.storage_type: redis
                    tools.orm_session.on: true
        sqlalchemy_engine:
            url: sqlite://
        email:
            host: localhost
            port: 1025
        """))
        path_file_mapping = {"/tmp/dev/app.yml": app_yml_file}
        self._stub_out_path_and_open(path_file_mapping)

        sys.argv = ("blueberrypy -C /tmp serve").split()
        main()

        app_config = load_yaml(app_yml_file.getvalue())
        controller_config = app_config["controllers"]['/api'].copy()
        controller = controller_config.pop("controller")
        controller_config["/"].update({"request.dispatch": controller})

        merged_app_config = cherrypy.tree.apps["/api"].config
        for k, v in controller_config.iteritems():
            self.assertEqual(v, merged_app_config[k])

        for k, v in app_config["global"].iteritems():
            self.assertEqual(v, merged_app_config["global"][k])

        for k, v in app_config["email"].iteritems():
            self.assertEqual(v, merged_app_config["email"][k])

        for k, v in app_config["sqlalchemy_engine"].iteritems():
            self.assertEqual(v, merged_app_config["sqlalchemy_engine"][k])

    def test_bind(self):
        self._setup_basic_app_config()

        # hack to get around problem with not being able to acquire a port
        # after listening on a different ip/port in a previous test 
        old_cherrypy_engine_start = cherrypy.engine.start
        def dummy_start(self):
            pass
        cherrypy.engine.start = dummy_start.__get__(cherrypy.engine, cherrypy.engine.__class__)

        sys.argv = "blueberrypy -C /tmp serve -b 0.0.0.0:9090".split()

        try:
            main()
            self.assertEqual(cherrypy.server.bind_addr, ("0.0.0.0", 9090))
        finally:
            cherrypy.engine.start = old_cherrypy_engine_start



import difflib
import inspect
import logging
import os.path
import textwrap
import warnings
import collections

import cherrypy

from yaml import load
try:
    from yaml import CLoader as Loader
except ImportError:
    from yaml import Loader

from blueberrypy.email import Mailer
from blueberrypy.exc import BlueberryPyNotConfiguredError, \
    BlueberryPyConfigurationError


logger = logging.getLogger(__name__)


class BlueberryPyConfiguration(object):

    def __init__(self, config_dir=None, app_config=None, logging_config=None,
                 webassets_env=None, environment=None):
        """Loads BlueberryPy configuration from `config_dir` if supplied.
        
        If `app_config` or `logging_config` or `webassets_env` are given, they
        will be used instead of the configuration files found from `config_dir`.
        
        If `environment` is given, it must be an existing CherryPy environment.
        If `environment` is `production`, and `config_dir` is given, the `prod`
        subdirectory will be searched for configuration files, otherwise the
        `dev` subdirectory` will be searched.
        
        Upon initialization of this configuration object, all the configuration
        will be validated for sanity and either BlueberryPyConfigurationError or
        BlueberryPyNotConfiguredError will be thrown if insane. For less severe
        configuration insanity cases, a warning will be emitted instead.
        
        :arg config_dir: a path, str
        :arg app_config: a CherryPy config, dict
        :arg logging_config: a logging config, dict
        :arg webassets_env: a webassets environment, webassets.Environment
        :arg environment: a CherryPy configuration environment, str
        """

        if config_dir is None:
            self.config_dir = config_dir = os.path.join(os.getcwdu(), "config")
        else:
            self.config_dir = config_dir = os.path.abspath(config_dir)

        if environment == "production":
            self.config_dir = config_dir = os.path.join(config_dir, "prod")
        elif environment == "test_suite" and os.path.exists(os.path.join(config_dir, "test")):
            self.config_dir = config_dir = os.path.join(config_dir, "test")
        else:
            self.config_dir = config_dir = os.path.join(config_dir, "dev")


        config_file_paths = {}
        app_yml_path = os.path.join(config_dir, "app.yml")
        logging_yml_path = os.path.join(config_dir, "logging.yml")
        bundles_yml_path = os.path.join(config_dir, "bundles.yml")

        if os.path.exists(app_yml_path):
            config_file_paths["app_yml"] = app_yml_path

        if os.path.exists(logging_yml_path):
            config_file_paths["logging_yml"] = logging_yml_path

        if os.path.exists(bundles_yml_path):
            config_file_paths["bundles_yml"] = bundles_yml_path

        self._config_file_paths = config_file_paths


        if "app_yml" in config_file_paths and not app_config:
            with open(config_file_paths["app_yml"]) as app_yml:
                self._app_config = load(app_yml, Loader)

        if "logging_yml" in config_file_paths and not logging_config:
            with open(config_file_paths["logging_yml"]) as logging_yml:
                self._logging_config = load(logging_yml, Loader)

        if "bundles_yml" in config_file_paths and not webassets_env:
            from webassets.loaders import YAMLLoader
            self._webassets_env = YAMLLoader(config_file_paths["bundles_yml"]).load_environment()


        if app_config:
            self._app_config = dict(app_config)

        if logging_config:
            self._logging_config = dict(logging_config)

        if webassets_env is not None:
            self._webassets_env = webassets_env

        self.validate()

        if environment == "weberror":
            self.setup_weberror_environment()

    @property
    def config_file_paths(self):
        if self._config_file_paths:
            sorted_kv_pairs = tuple(((k, self._config_file_paths[k]) for k in sorted(self._config_file_paths.iterkeys())))
            paths = collections.namedtuple("config_file_paths", [e[0] for e in sorted_kv_pairs])
            return paths(*[e[1] for e in sorted_kv_pairs])

    @property
    def project_metadata(self):
        return self.app_config["project_metadata"]

    @property
    def use_logging(self):
        return self.app_config.get("global", {}).get("engine.logging.on", False)

    @property
    def use_redis(self):
        if self.controllers_config:
            for _, controller_config in self.controllers_config.iteritems():
                controller_config = controller_config.copy()
                controller_config.pop("controller")
                for path_config in controller_config.itervalues():
                    if path_config.get("tools.sessions.storage_type") == "redis":
                        return True
        return False

    @property
    def use_sqlalchemy(self):
        return self.app_config.get("global", {}).get("engine.sqlalchemy.on", False)

    @property
    def use_jinja2(self):
        return "jinja2" in self.app_config

    @property
    def use_webassets(self):
        return self.use_jinja2 and self.app_config["jinja2"].get("use_webassets", False)

    @property
    def use_email(self):
        return "email" in self.app_config

    @property
    def controllers_config(self):
        return self.app_config.get("controllers")

    @property
    def app_config(self):
        return self._app_config

    @property
    def logging_config(self):
        return getattr(self, "_logging_config", None)

    @property
    def webassets_env(self):
        return getattr(self, "_webassets_env", None)

    @property
    def jinja2_config(self):
        if self.use_jinja2:
            conf = self.app_config["jinja2"].copy()
            conf.pop("use_webassets", None)
            return conf

    @property
    def sqlalchemy_config(self):
        if self.use_sqlalchemy:
            if "sqlalchemy_engine" in self.app_config:
                saconf = self.app_config["sqlalchemy_engine"].copy()
                return {"sqlalchemy_engine": saconf}
            else:
                return dict([(k, v) for k, v in self.app_config.iteritems()
                             if k.startswith("sqlalchemy_engine")])

    @property
    def email_config(self):
        return self.app_config.get("email")

    def setup_weberror_environment(self):
        """Returns a new copy of this configuration object configured to run
        under the weberror environment and ensure the weberror environment
        is created for cherrypy's config object."""

        try:
            from weberror.evalexception import EvalException
        except ImportError:
            warnings.warn("WebError not installed")
            return

        cherrypy._cpconfig.environments["weberror"] = {
            "log.wsgi": True,
            "request.throw_errors": True,
            "log.screen": False,
            "engine.autoreload_on": False
        }

        def remove_error_options(section):
            section.pop("request.handler_error", None)
            section.pop("request.error_response", None)
            section.pop("tools.err_redirect.on", None)
            section.pop("tools.log_headers.on", None)
            section.pop("tools.log_tracebacks.on", None)

            for k in section.copy().iterkeys():
                if k.startswith("error_page.") or \
                        k.startswith("request.error_page."):
                    section.pop(k)

        for section_name, section in self.app_config.iteritems():
            if section_name.startswith("/") or section_name == "global":
                remove_error_options(section)

        wsgi_pipeline = []
        if "/" in self.app_config:
            wsgi_pipeline = self.app_config["/"].get("wsgi.pipeline", [])
        else:
            self.app_config["/"] = {}

        wsgi_pipeline.insert(0, ("evalexc", EvalException))

        self.app_config["/"]["wsgi.pipeline"] = wsgi_pipeline

    def validate(self):
        # no need to check for cp config, which will be checked on startup

        if not hasattr(self, "_app_config") or not self.app_config:
            raise BlueberryPyNotConfiguredError("BlueberryPy application configuration not found.")

        if self.use_sqlalchemy and not self.sqlalchemy_config:
            raise BlueberryPyNotConfiguredError("SQLAlchemy configuration not found.")

        if self.use_webassets:
            if self.webassets_env is None:
                raise BlueberryPyNotConfiguredError("Webassets configuration not found.")
            elif len(self.webassets_env) == 0:
                raise BlueberryPyNotConfiguredError("No bundles found in webassets env.")

        if self.use_jinja2 and not self.jinja2_config:
            raise BlueberryPyNotConfiguredError("Jinja2 configuration not found.")

        if self.use_logging and not self.logging_config:
            warnings.warn("BlueberryPy application-specific logging "
                          "configuration not found. Continuing without "
                          "BlueberryPy's logging plugin.")

        if self.use_email:
            if not self.email_config:
                warnings.warn("BlueberryPy email configuration is empty.")
            else:
                mailer_ctor_argspec = inspect.getargspec(Mailer.__init__)
                argnames = frozenset(mailer_ctor_argspec.args[1:])
                for key in self.email_config.iterkeys():
                    if key not in argnames:
                        closest_match = difflib.get_close_matches(key, argnames, 1)
                        closest_match = (closest_match and " Did you mean %r?" % closest_match[0]) or ""
                        warnings.warn(("Unknown key %r found for [email]." % key) + closest_match)

        if not self.controllers_config:
            raise BlueberryPyConfigurationError("You must declare at least one controller.")
        else:
            for script_name, section in self.controllers_config.iteritems():
                controller = section.get("controller")
                if controller is None:
                    raise BlueberryPyConfigurationError("You must define a controller in the [controllers][%s] section." % script_name)
                elif isinstance(controller, cherrypy.dispatch.RoutesDispatcher):
                    if not controller.controllers:
                        warnings.warn("Controller %r has no connected routes." % script_name)
                else:
                    for member_name, member_obj in inspect.getmembers(controller):
                        if member_name == "exposed" and member_obj:
                            break
                        elif (hasattr(member_obj, "exposed") and
                              member_obj.exposed == True):
                            break
                    else:
                        raise warnings.warn("Controller %r has no exposed method." % script_name)



__version__ = "0.5.3"

import logging
import sys

logger = logging.getLogger(__name__)

logger.propagate = False
logger.setLevel(logging.INFO)
handler = logging.StreamHandler(sys.stdout)
handler.setLevel(logging.INFO)
logger.addHandler(handler)
